/*
////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////
*/

#ifndef ICMODELLIB_H
#define ICMODELLIB_H

/*
// \brief Interface of the ARTIST model library
//
//  May be used from C or C++
*/

#ifdef __cplusplus
extern "C" {
#endif
  /* Exploration: */
  /* Models: */
  int getNumModels();  
  char** getModelIDs();
  int isRegisteredModel(const char* modelID);
  char* getModelName(const char* modelID);
  char* getModelDesc(const char* modelID);
  char* getModelBibref(const char* modelID);

  int getNumParams(const char* modelID);  
  char** getParamIDs(const char* modelID);
  int isRegisteredParam(const char* modelID, const char* paramID);
  char* getParamType(const char* modelID, const char* paramID);
  char* getParamName(const char* modelID, const char* paramID);
  char* getParamDesc(const char* modelID, const char* paramID);
  char* getParamUnit(const char* modelID, const char* paramID);
  int getParamDefValInt(const char* modelID, const char* paramID);
  double getParamDefValDouble(const char* modelID, const char* paramID);
  char* getParamDefValString(const char* modelID, const char* paramID);
  int getNumParamEnumValues(const char* modelID, const char* paramID);
  double* getParamEnumValues(const char* modelID, const char* paramID);
  unsigned getParamEnumDefIndex(const char* modelID, const char* paramID);


  /* Results: */
  int getNumResults();
  char** getResultIDs();
  int isRegisteredResult(const char* resultID);
  char* getResultName(const char* resultID);
  char* getResultDesc(const char* resultID);
  char* getResultUnit(const char* resultID);
  char* getResultType(const char* resultID);

  int getNumModelResults(const char* modelID);
  char** getModelResultIDs(const char* modelID);
  int isResultModel(const char* resultID, const char* modelID);

  /* Functions: */
  int getNumFunctions();
  char** getFunctionIDs();
  int isRegisteredFunction(const char* functionID);
  char* getFunctionName(const char* functionID);
  char* getFunctionDesc(const char* functionID);
  char* getFunctionType(const char* functionID);

  int getNumFunctionResults(const char* functionID);
  char** getFunctionResultIDs(const char* functionID);
  int getNumResultFunctions(const char* resultID);
  char** getResultFunctionIDs(const char* resultID);
  int isResultFunction(const char* resultID, const char* functionID);

  int getNumFunctionParams(const char* functionID);
  char** getFunctionParamIDs(const char* functionID);
  int isRegisteredFunctionParam(const char* functionID, const char* paramID);
  char* getFunctionParamType(const char* functionID, const char* paramID);
  char* getFunctionParamName(const char* functionID, const char* paramID);
  char* getFunctionParamDesc(const char* functionID, const char* paramID);
  int getFunctionParamDefValInt(const char* functionID, const char* paramID);
  double getFunctionParamDefValDouble(const char* functionID, const char* paramID);
  char* getFunctionParamDefValString(const char* functionID, const char* paramID);
  int getNumFunctionParamEnumValues(const char* functionID, const char* paramID);
  double* getFunctionParamEnumValues(const char* functionID, const char* paramID);
  unsigned getFunctionParamEnumDefIndex(const char* functionID, const char* paramID);


  /* Configuration: */
  /* Model: */
  void setCurrentModel(const char* modelID);
  void unsetCurrentModel();
  int isSetCurrentModel();
  char* getCurrentModelID();

  void setParamDouble(const char* paramID, double val);
  void setParamInt(const char* paramID, int val);
  void setParamString(const char* paramID, const char* val);
  void setParamEnumIndex(const char* paramID, const unsigned index);

  double getParamDouble(const char* paramID);
  int getParamInt(const char* paramID);
  char* getParamString(const char* paramID);
  unsigned getParamEnumIndex(const char* paramID);

  /* Functions: */
  void setFunction(const char* resultID, const char* functionID);
  void unsetFunction(const char* resultID);
  char* getFunctionID(const char* resultID);

  void setFunctionParamDouble(const char* resultID, const char* paramID, double val);
  void setFunctionParamInt(const char* resultID, const char* paramID, int val);
  void setFunctionParamString(const char* resultID, const char* paramID, const char* val);
  void setFunctionParamEnumIndex(const char* resultID, const char* paramID, const unsigned index);

  double getFunctionParamDouble(const char* resultID, const char* paramID);
  int getFunctionParamInt(const char* resultID, const char* paramID);
  char* getFunctionParamString(const char* resultID, const char* paramID);
  unsigned getFunctionParamEnumIndex(const char* resultID, const char* paramID);

  /* Identification of Tdust with Temperature: */
  void setTdustIdentTemp();
  void unsetTdustIdentTemp();
  int isTdustIdentTemp();

  /* Identification of Temperature with Tdust: */
  void setTempIdentTdust();
  void unsetTempIdentTdust();
  int isTempIdentTdust();


  /* Results: */
  int isCurrentResultModel(const char* resultID);
  int isCurrentResultFunction(const char* resultID);

  int isConfigurationComplete();

  void finalizeConfiguration();
  int isFinalizedConfiguration();

  /* Usage: */
  double modellibDensity(double x, double y, double z);
  double modellibTemperature(double x, double y, double z);
  double modellibAbundance(double x, double y, double z);
  double modellibTdust(double x, double y, double z);
  double modellibDoppler(double x, double y, double z);
  void modellibVelocity(double x, double y, double z, double* v);
  void modellibBmag(double x, double y, double z, double* b);
  
#ifdef __cplusplus
};
#endif

#endif //ICMODELLIB_H
