////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012, Reinhold Schaaf
// Argelander-Institut fuer Astronomie
// Universitaet Bonn
// rschaaf@astro.uni-bonn.de
//
// and the ARTIST team.
//
// For further info about copyright and license see included license.txt file.
//
////////////////////////////////////////////////////////////////////////////////

/**
 * @file icppModellib.h
 * @author rschaaf
 *
 * Interface to be used from C++
 * Header file
 *
 */

// Todo: Alle const checken
// Todo: Defaultwerte fuer Funktionsparameter Schwaeche: Functions (und Models) kennen ihre eigenen Parameter nicht. Nur in functionregistry und modelregistry bekannt
// Todo: Downcasting von IProvider nach Model und IDensityProvider etc. muss gecheckt werden

#ifndef ICPPMODELLIB_H
#define ICPPMODELLIB_H

#include <string>
#include <set>
#include <vector>

using namespace std;

////////////////////////////////////////////////////////////////////
//! Functions that allow the exploration of the model library.
/**
 * @name Exploration
 */
//@{

// Exploration:

// Models:
set<string> getModelIDs();
bool isRegisteredModel(const string modelID);
string getModelName(const string modelID);
string getModelDesc(const string modelID);
string getModelBibref(const string modelID);

set<string> getParamIDs(const string modelID);
bool isRegisteredParam(const string modelID, string paramID);
string getParamName(const string modelID, const string paramID);
string getParamDesc(const string modelID, const string paramID);
string getParamType(const string modelID, const string paramID);
string getParamUnit(const string modelID, const string paramID);
int getParamDefValInt(const string modelID, const string paramID);
double getParamDefValDouble(const string modelID, const string paramID);
string getParamDefValString(const string modelID, const string paramID);
vector<double> getParamEnumValues(const string modelID, const string paramID);
unsigned getParamEnumDefIndex(const string modelID, const string paramID);


// Results:
set<string> getResultIDs();
bool isRegisteredResult(const string resultID);
string getResultName(const string resultID);
string getResultDesc(const string resultID);
string getResultUnit(const string resultID);
string getResultType(const string resultID);

set<string> getModelResultIDs(const string modelID);
bool isResultModel(const string resultID, const string modelID);

// Functions:
set<string> getFunctionIDs();
bool isRegisteredFunction(const string functionID);
string getFunctionName(const string functionID);
string getFunctionDesc(const string functionID);
string getFunctionType(const string functionID);

set<string> getFunctionResultIDs(const string functionID);
set<string> getResultFunctionIDs(const string resultID);
bool isResultFunction(const string resultID, const string functionID);

set<string> getFunctionParamIDs(const string functionID);
bool isRegisteredFunctionParam(const string functionID, string paramID);
string getFunctionParamName(const string functionID, const string paramID);
string getFunctionParamDesc(const string functionID, const string paramID);
string getFunctionParamType(const string functionID, const string paramID);
int getFunctionParamDefValInt(const string functionID, const string paramID);
double getFunctionParamDefValDouble(const string functionID, const string paramID);
string getFunctionParamDefValString(const string functionID, const string paramID);
vector<double> getFunctionParamEnumValues(const string functionID, const string paramID);
unsigned getFunctionParamEnumDefIndex(const string functionID, const string paramID);

//@}

////////////////////////////////////////////////////////////////////
//! Functions for setting up of the model library.
/**
 * @name Setup
 */
//@{

// Setup:

// Model:
void setCurrentModel(const string modelID);
void unsetCurrentModel();
bool isSetCurrentModel();
string getCurrentModelID();

void setParamDouble(const string paramID, double val);
void setParamInt(const string paramID, int val);
void setParamString(const string paramID, const string val);
void setParamEnumIndex(const string paramID, unsigned index);

double getParamDouble(const string paramID);
int getParamInt(const string paramID);
string getParamString(const string paramID);
unsigned getParamEnumIndex(const string paramID);

// Functions:
void setFunction(const string resultID, const string functionID);
void unsetFunction(const string resultID);
const string getFunctionID(const string resultID);

void setFunctionParamDouble(const string resultID, const string paramID, double val);
void setFunctionParamInt(const string resultID, const string paramID, int val);
void setFunctionParamString(const string resultID, const string paramID, string val);
void setFunctionParamEnumIndex(const string resultID, const string paramID, unsigned index);

double getFunctionParamDouble(const string resultID, const string paramID);
int getFunctionParamInt(const string resultID, const string paramID);
string getFunctionParamString(const string resultID, const string paramID);
unsigned getFunctionParamEnumIndex(const string resultID, const string paramID);

// Identification of Tdust with Temperature:
void setTdustIdentTemp();
void unsetTdustIdentTemp();
const bool isTdustIdentTemp();

// Identification of Temperature with Tdust:
void setTempIdentTdust();
void unsetTempIdentTdust();
const bool isTempIdentTdust();

// Results:
const bool isCurrentResultModel(const string resultID);
const bool isCurrentResultFunction(const string resultID);

const bool isConfigurationComplete();

void finalizeConfiguration();
bool isFinalizedConfiguration();

//@}

////////////////////////////////////////////////////////////////////
//! Functions for using the model library.
/**
 * @name Usage
 */
//@{

// Usage:

double density(double x, double y, double z);
double temperature(double x, double y, double z);
double abundance(double x, double y, double z);
double t_dust(double x, double y, double z);
double doppler(double x, double y, double z);
vector<double> velocity(double x, double y, double z);
vector<double> bmag(double x, double y, double z);

//@}

#endif //ICPPMODELLIB_H
