# runLime.py
#
# Script to demonstrate the complete course of selecting and
# setting up the Current Model, defining input parameters and 
# images for LIME, and running LIME.
#
# The script selects the Current Model, sets its Parameters,
# and links Functions to Results that are not provided by the Model.
# After the Model Library is set up thus, LIME's input parameters
# and the parameters for two images are defined. Then LIME is run.
#
# Note that in this simple script no status of LIME's progress can be
# accessed.

import modellib
import lime

import time
import os
import math

artistdir = os.getenv("ARTIST_ROOT", "..")

AU = 1.49598e11    # AU to m
PC = 3.08568025e16 # PC to m

t0 = time.time()

# Select the Current Model:
modellib.setCurrentModel('CG97')

# Set all parameters:
modellib.setParamDouble('Mstar', 0.5)    # Msun
modellib.setParamDouble('Rstar', 2.0)    # Rsun
modellib.setParamDouble('Tstar', 4000.)  # K
modellib.setParamDouble('bgdens', 1.e-4) # 1/cm^3
modellib.setParamDouble('hph', 4.0)
modellib.setParamDouble('plsig1', -1.0)
modellib.setParamDouble('rin', 1.0)      # AU
modellib.setParamDouble('rout', 100.0)   # AU
modellib.setParamDouble('sig0', 0.01)    # g/cm^2

# Model CG97 does not provide the results 'abundance',
# 'doppler', 'temperature, and 'bmag'. Hence these
# Results have to be provided by Functions

# Constant abundance (1.e-9):
modellib.setFunction('abundance', 'scalarConst')
modellib.setFunctionParamDouble('abundance', 'val', 1.e-9)

# Constant doppler (100 m/s):
modellib.setFunction('doppler', 'scalarConst')
modellib.setFunctionParamDouble('doppler', 'val', 100.)

# Zero bmag:
modellib.setFunction('bmag', 'vectorConstR')
modellib.setFunctionParamDouble('bmag', 'val', 0.)

# Set temperature identical to t_dust:
modellib.setTempIdentTdust()

# Done setting up Model Library:
modellib.finalizeConfiguration()

# Set input parameters for lime:
par = lime.createInputPars()

par.radius      = 800 * AU
par.minScale    = 1. * AU
par.pIntensity  = 1000
par.sinkPoints  = 1000

par.dust        = os.path.join(artistdir, "data/jena_thin_e6.tab")
par.moldatfile  = os.path.join(artistdir, "data/hco+.dat")
par.outputfile  = "outfile.pop"
par.gridfile    = "grid.vtk"

# Define the images:
images = []

# Image #0: J=2-1
img = lime.createImage()

img.trans       = 2
img.filename    = "image2.fits"

img.pxls        = 101
img.imgres      = 0.02
img.distance    = 140 * PC
img.theta       = math.pi * 30. / 180.
img.phi         = 0.0
img.source_vel  = 0.0
img.unit        = 0

img.nchan       = 60
img.velres      = 100.

images.append(img)

# Image #1: J=3-2
img = lime.createImage()

img.trans       = 3
img.filename    = "image3.fits"

img.pxls        = 101
img.imgres      = 0.02
img.distance    = 140 * PC
img.theta       = math.pi * 30. / 180.
img.phi         = 0.0
img.source_vel  = 0.0
img.unit        = 0

img.nchan       = 60
img.velres      = 100.

images.append(img)


# Run LIME:
lime.setSilent(False)

print "Running LIME:"
try:
  lime.runLime(par, images)
except Exception, e:
  print "Exception:", e

t1 = time.time()
print "Runtime: %ds" % (t1 - t0)
