################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
import os, sys
"""
Class for convert the output of different code to an ARTIST table format
Currently supported codes:
"""

def getVarNames(codeName=None):
    """
    Returns the list of variables that can be imported from a given code

    INPUT:
        codeName - Name of the code 
    """

    if codeName=='radmc3d':
        return ['tdust', 'temperature', 'velocity','dustdens', 'density']  
    if codeName=='radmc':
        return ['tdust', 'dustdens', 'density']
    if codeName=='artist':
        return ['abundance', 'bmag', 'density', 'doppler', 'tdust', 'temperature', 'velocity']
    if codeName==None:
        print 'ERROR'
        print 'codeName is not given'
        print 'SYNTAX:'
        print 'ArtistTable.getVarNames(codeName="")'
        return

def getCodeNames():
    """
    Returns the list of supported code names
    """

    return ['radmc3d', 'radmc']

def importTable(tableDir='',codeName='',varName='',outDir=''):
    """
    Imports data to ARTIST format

    INPUT:
    ------
        tableDir - Path to the directory where the data, written by external code, can be found
        codeName - Name of the code whose output should be imported
        varName  - Name of the physical variable that should be imported
        
    OPTIONS:
        outDir - Target directory where the ARTIST table should be written 
                 If omitted the current working directory will be used
    """
    

    if tableDir=='': tableDir='./'
    if codeName=='':
        print ' ERROR ' 
        print ' codeName is not specified'
        return
    if varName=='':
        print ' ERROR ' 
        print ' varName is not specified'
        return

    home = os.getcwd()
    try:
        os.chdir(tableDir)
    except:
        print 'ERROR'
        print 'Cannot access '+tableDir
        return
 


    table = DataTable()
    dum = table.readData(codeName, varName)
    if dum>0: return [dum, '', '']
    os.chdir(home)


    if outDir!='':
        try: 
            os.chdir(outDir)
        except:
            print 'ERROR'
            print 'Cannot access '+outDir

        table.writeTable()
        os.chdir(home)
    else:
        os.chdir(home)
        table.writeTable()
    
    return [0, table.importedFName, table.tableFName] 

class DataTable():
    """
    ARTIST Data Table class

    ATTRIBUTES:
    -----------
    act_dim         - Three elements list indicating which dimension is active (i.e. for a 1D spherical coordinate system act_dim=[1,0,0])
    cellCenter      - A bool variable set to True if the grid is cell centered, while for corner-centered grids it is set to False 
    codeName        - Name of the external code from which the data has been imported 
    data            - A numpy.ndarray variable containing the physical variable
    gridStyle       - If 0 the grid is regular, if 1 the grid is irregular 
    gridType        - For regular grids: 0-Cartesian, 1-Cylindrical, 2-Spherical, for irregular grids: 0-AMR, 1-Delaunay
    importedFName   - Name of the file from which the data has been imported (external code) 
    nx              - Number of cell centers in the first spatial coordinate (x-Cartesian, R-Cylindrical, r-Spherical)
    nxi             - Number of cell interfaces in the first spatial coordiante (x-Cartesian, R-Cylindrical, r-Spherical)
    ny              - Number of cell centers in the second spatial coordinate (y-Cartesian, z-Cylindrical, theta-Spherical)
    nyi             - Number of cell interfaces in the second spatial coordiante  (y-Cartesian, z-Cylindrical, theta-Spherical)
    nz              - Number of cell centers in the third spatial coordinate (z-Cartesian, phi-Cylindrical, phi-Spherical)
    nzi             - Number of cell interfaces in the third spatial coordiante  (z-Cartesian, phi-Cylindrical, phi-Spherical)
    x               - Grid points in the first spatial coordinate
    y               - Grid points in the second spatial coordinate
    varName         - Name of the variable stored 
    tableFName      - Name of the file into which the variable should be written (ARTIST Table file format)
    z               - Grid points in the third spatial coordinate

    """

    def __init__(self):

        self.__version__ = 'v1.0beta'
        self.varName = ''
        self.gridStyle = 0  
        self.gridType = 0  
                          

        self.cellCenter = False
        self.nx = 0
        self.ny = 0
        self.nz = 0
        self.nxi = 0
        self.nyi = 0
        self.nzi = 0
        self.x = []
        self.y = []
        self.z = []
        self.xi = []
        self.yi = []
        self.zi = []
        self.act_dim = [] 
        self.data = []
        self.codeName = ''
        self.importedFName = ''
        self.tableFName = ''

    def getVarNames(self):
        """
        Returns the list of valid variable names that can be imported
        """
        return ['abundance', 'bmag', 'density', 'doppler', 'tdust', 'temperature', 'velocity']

    def writeTable(self):
        """
        Writes data into an ARTIST table format 
        """
        self._writeTable_artist()


    def readData(self, codeName='', varName='', tableFName=''):
        """
        Reads data written by an external code

        INPUT:
            codeName - Name of the code whose format should be read
                        for supported codes see getCodeNames()
            varName - Name of the variable that should be read
                        for supported codes see getVarNames(codeName="")
            tableFName - Name of the table to be read (only for codeName='artist')
        """

        if codeName.lower()=='radmc3d':
            dum = self._readData_radmc3d(varName=varName)
            self.codeName = 'radmc3d'
        elif codeName.lower()=='radmc':
            dum = self._readData_radmc(varName=varName)
            self.codeName = 'radmc'
        elif codeName.lower()=='artist':
            dum = self._readData_artist(tableFName=tableFName)
            self.codeName = 'artist'
        else:
            print ' Unsupported code: '
            print codeName
            print ' For list the currently supported codes use getCodeNames()'
            return -1

        return dum
    def _writeTable_artist(self):
        """
        Function to write data in artist format
        """

        if self.gridStyle==0:
            if self.importedFName=='':
                if self.varName == 'abundance'   : self.tableFName='abundance.ARTIST.tab'
                if self.varName == 'bmag'        : self.tableFName='bmag.ARTIST.tab'
                if self.varName == 'density'     : self.tableFName='density.ARTIST.tab'
                if self.varName == 'doppler'     : self.tableFName='doppler.ARTIST.tab'
                if self.varName == 'tdust'       : self.tableFName='tdust.ARTIST.tab'
                if self.varName == 'temperature' : self.tableFName='temperature.ARTIST.tab'
                if self.varName == 'velocity'    : self.tableFName='velocity.ARTIST.tab'

            fileName = self.tableFName
            
            try:
                wfile=open(fileName, 'w')
            except:
                print 'ERROR'
                print fileName+' cannot be created'
                return

            wfile.write("%s\n"%self.varName)
            wfile.write("%d\n"%self.gridStyle)
            wfile.write("%d\n"%self.gridType)
            if self.cellCenter:
                wfile.write("%d\n"%0)
            else:
                wfile.write("%d\n"%1)
            if self.act_dim[0]==0:
                nx = 0
            else:
                nx = self.nxi
            if self.act_dim[1]==0:
                ny = 0 
            else:
                ny = self.nyi
            if self.act_dim[2]==0:
                nz = 0
            else:
                nz = self.nzi

            wfile.write("%d %d %d\n"%(nx,ny,nz))
            wfile.write("\n")


                    
            if self.cellCenter:
                if self.act_dim[0]!=0 : 
                    for ix in range(self.nx): wfile.write("%.9e\n"%self.x[ix])
                    wfile.write("\n")
                if self.act_dim[1]!=0 : 
                    for iy in range(self.ny): wfile.write("%.9e\n"%self.y[iy])
                    wfile.write("\n")
                if self.act_dim[2]!=0 : 
                    for iz in range(self.nz): wfile.write("%.9e\n"%self.z[iz])
                    wfile.write("\n")
            else:
                if self.act_dim[0]!=0 : 
                    for ix in range(self.nxi): wfile.write("%.9e\n"%self.xi[ix])
                    wfile.write("\n")
                if self.act_dim[1]!=0 : 
                    for iy in range(self.nyi): wfile.write("%.9e\n"%self.yi[iy])
                    wfile.write("\n")
                if self.act_dim[2]!=0 : 
                    for iz in range(self.nzi): wfile.write("%.9e\n"%self.zi[iz])
                    wfile.write("\n")


            if len(self.data.shape)==3:
                for ix in range(self.nx):
                    for iy in range(self.ny):
                        for iz in range(self.nz):
                            wfile.write("%.9e\n"%self.data[ix,iy,iz])

            elif len(self.data.shape)==4:
                if self.data.shape[3]==3:
                    for ix in range(self.nx):
                        for iy in range(self.ny):
                            for iz in range(self.nz):
                                wfile.write("%.9e %.9e %.9e\n"%(self.data[ix,iy,iz,0],self.data[ix,iy,iz,1],self.data[ix,iy,iz,2]))
                else:
                    print ' ERROR '
                    print ' Unknown data type with array shape : '
                    print self.data.shape

            else:
                print ' ERROR '
                print ' Unknown data type with array shape : '
                print self.data.shape

 

        else:
            print ' ERROR '
            print ' Unstructured grids (AMR, Delaunay) are not yet supported '
            return

    def _readData_artist(self, tableFName=''):
        """
        Reads data from ARTIST Table format

        INPUT:

            tableFName - Name of the file to be imported

        """

        try:
            rfile = open(tableFName, 'r')
        except:
            print 'ERROR'
            print tableFName+' cannot be opened'
            return 1

        dum = rfile.readline().strip()
        allowedVarNames = getVarNames(codeName='artist')
        if not allowedVarNames.__contains__(dum):
            rfile.close()
            print 'ERROR'
            print tableFName + ' contains an unknown variable : '+dum
            return 2
        else:
            varName = dum

        dum = int(rfile.readline().strip())
        if (dum<0)|(dum>1):
            rfile.close()
            print 'ERROR'
            print 'Unknown grid style : ', dum
            return 3
        else:
            gridStyle=dum

        dum = int(rfile.readline().strip())
        if gridStyle==0:
            if (dum<0)|(dum>2):
                rfile.close()
                print 'ERROR'
                print 'Unknown gryd type : ', dum
                return 4
            else:
                gridType=dum
        elif gridStyle==1:
            if (dum<0)|(dum>1):
                rfile.close()
                print 'ERROR'
                print 'Unknown gryd type : ', dum
                return 4
            else:
                gridType=dum

        dum = int(rfile.readline().strip())
        if (dum<0)|(dum>1):
            rfile.close()
            print 'ERROR'
            print 'Unknown grid (cell center vs corner center) : ', dum
            return 5
        else:
            if dum==1: 
                cellCenter=False
            else:
                cellCenter=True

        dum = rfile.readline().split()
        if len(dum)!=3:
            print 'ERROR' 
            print 'The number of grid points in all three dimensions should be specified'
            print 'Unused dimensions (e.g. due to symmetry) should be indicated by 0 in that grid axis'
            return 6
        else:
            if not cellCenter:
                nxi = max([int(dum[0]),1])
                nyi = max([int(dum[1]),1])
                nzi = max([int(dum[2]),1])
            
                nx = max([nxi-1, 1])
                ny = max([nyi-1, 1])
                nz = max([nzi-1, 1])

            else:
                nx = max([int(dum[0]),1])
                ny = max([int(dum[1]),1])
                nz = max([int(dum[2]),1])
            
                nxi = min([nx+1, 1])
                nyi = min([ny+1, 1])
                nzi = min([nz+1, 1])

           
        act_dim = [0,0,0]
        if nx>1: act_dim[0]=1
        if ny>1: act_dim[1]=1
        if nz>1: act_dim[2]=1

        if nx>1:
            dum = rfile.readline()
            if not cellCenter:
                xi = np.zeros(nxi, dtype=np.float64)
                for ix in range(nxi):
                    dum = rfile.readline()
                    xi[ix] = float(dum)

                x = 0.5*(xi[:-1] + xi[1:])

            else:
                x = np.zeros(nx, dtype=np.float64)
                for ix in range(nx):
                    x[ix] = float(rfile.readline())
                
                xi = np.zeros(nxi,dtype=np.float64)
                xi[0] = x[0] + (x[0] - x[1])
                xi[1:-1] = 0.5 * (x[:-1] + x[1:])
                xi[nxi] = x[nx-1] + (x[nx-1] - x[nx-2])
        else:
            x = -1
            xi = -1
            

        if ny>1:
            dum = rfile.readline()
            if not cellCenter:
                yi = np.zeros(nyi, dtype=np.float64)
                for iy in range(nyi):
                    dum = rfile.readline()
                    print iy, dum
                    yi[iy] = float(dum)

                y = 0.5*(yi[:-1] + yi[1:])

            else:
                y = np.zeros(ny, dtype=np.float64)
                for iy in range(ny):
                    y[iy] = float(rfile.readline())
                
                yi = np.zeros(nyi,dtype=np.float64)
                yi[0] = y[0] + (y[0] - y[1])
                yi[1:-1] = 0.5 * (y[:-1] + y[1:])
                yi[nyi] = y[ny-1] + (y[ny-1] - y[ny-2])
        else:
            y = -1
            yi = -1

        if nz>1:
            dum = rfile.readline()

            if not cellCenter:
                zi = np.zeros(nzi, dtype=np.float64)
                for iz in range(nzi):
                    zi[iz] = float(rfile.readline())

                z = 0.5*(zi[:-1] + zi[1:])

            else:
                z = np.zeros(nz, dtype=np.float64)
                for iz in range(nz):
                    z[iz] = float(rfile.readline())
                
                zi = np.zeros(nzi,dtype=np.float64)
                zi[0] = z[0] + (z[0] - z[1])
                zi[1:-1] = 0.5 * (z[:-1] + z[1:])
                zi[nzi] = z[nz-1] + (z[nz-1] - z[nz-2])
        else:
            z = -1
            zi = -1

        dum = rfile.readline()
        if (varName=='velocity')|(varName=='bmag'):
            data = np.zeros([nx,ny,nz,3], dtype=np.float64)
            for ix in range(nx):
                for iy in range(ny):
                    for iz in range(nz):
                        dum = rfile.readline().split()
                        data[ix,iy,iz,0] = float(dum[0])
                        data[ix,iy,iz,1] = float(dum[1])
                        data[ix,iy,iz,2] = float(dum[2])

        else:
            data = np.zeros([nx,ny,nz], dtype=np.float64)
            nlpx = max([nx,1])
            nlpy = max([ny,1])
            nlpz = max([nz,1])

            for ix in range(nx):
                for iy in range(ny):
                    for iz in range(nz):
                        dum = rfile.readline()
                        data[ix,iy,iz] = float(dum)

        rfile.close()



        self.varName = varName
        self.gridStyle = gridStyle
        self.gridType = gridType
        self.cellCenter = cellCenter
        self.nx = nx
        self.nxi = nxi
        self.x  = np.array(x)
        self.xi = np.array(xi)
        self.ny = ny
        self.nyi = nyi
        self.y  = np.array(y)
        self.yi = np.array(yi)
        self.nz = nz
        self.nzi = nzi
        self.z  = np.array(z)
        self.zi = np.array(zi)
        self.act_dim = act_dim
        self.data = np.array(data)
        self.codeName = 'artist'
        self.importedFName = tableFName



        return 0


    def _readData_radmc(self, varName=''):
        """
        Reads the data from RADMC-3D

        INPUT:
            varName - Name of the variable to be imported 
                        for available variables see getVarNames("radmc")
        """

        self.gridStyle = 0
        self.gridType  = 2
        self.cellCenter = False
        self.act_dim   = [1,1,0]

        try:
            rfile = open('radius.inp', 'r')
        except:
            print 'ERROR'
            print ' radius.inp cannot be opened'
            return 1

        self.nx = int(rfile.readline())
        self.x  = zeros(self.nx, dtype=float)
        dum = rfile.readline() 
        for ix in range(self.nx):
            self.x[ix] = float(rfile.readline())
        rfile.close()

        self.xi = np.sqrt(self.x[1:] * self.x[:-1])
        self.xi = np.append(self.xi[0]*self.xi[0]/self.xi[1], self.xi)
        self.xi = np.append(self.xi, self.x[self.nx-1]*self.x[self.nx-1]/self.x[self.nx-2])
        self.nxi = self.xi.shape[0] 
        self.x = self.x*0.01
        self.xi = self.xi*0.01


        try:
            rfile = open('theta.inp', 'r')
        except:
            print 'ERROR'
            print 'theta.inp cannot be opened'
            return 2

        dum = rfile.readline().split()
        imirt = int(dum[1])
        if imirt==1:
            self.ny = int(dum[0])*2
            self.y = zeros(self.ny, dtype=float)
            dum = rfile.readline()
            for iy in range(self.ny/2):
                self.y[iy] = float(rfile.readline())
            
            self.y[self.ny/2:] = pi - self.y[:self.ny/2][::-1]
        else:
            self.ny = int(dum[0])
            self.y = zeros(self.ny, dtype=float)
            dum = rfile.readline()
            for iy in range(self.ny):
                self.y[iy] = float(rfile.readline())

        rfile.close()

        self.yi = 0.5 * (self.y[1:] + self.y[:-1])
        self.yi = np.append(2.0*self.yi[0]-self.yi[1], self.yi)
        self.yi = np.append(self.yi, 2.0*self.y[self.ny-1]-self.y[self.ny-2])
        self.nyi = self.yi.shape[0] 


        self.nz = 1
        self.nzi = 1
        


        if varName=='tdust':
            self.varName = 'tdust'
            try:
                rfile = open('dusttemp_final.dat', 'r')
            except:
                print 'ERROR'
                print 'dusttemp_final.dat cannot be opened'
                return 6
            dum = rfile.readline().split()
            ndust = int(dum[0])
            nr    = int(dum[1])
            nt    = int(dum[2])
            
            if int(dum[3])!=imirt:
                print 'ERROR'
                print 'Dust temperature is defined in a different theta grid than given in theta.inp'
                print 'Theta mirroring is different in dusttemp_final.dat and theta.inp'
                rfile.close()
                return 3

            if ndust>1:
                print 'WARNING'
                print 'Multiple-dust species are not yet supported'
                print 'I take the variables of the first dust species'

            if nr!=self.nx:
                print 'ERROR'
                print 'Number of radial grid points is different in dusttemp_final.dat and radius.inp'
                rfile.close()
                return 4

            if imirt==1:
                if nt!=self.ny/2:
                    print 'ERROR'
                    print 'Number of theta grid points is different in dusttemp_final.dat and theta.inp'
                    rfile.close()
                    return 5
            else:
                if nt!=self.ny:
                    print 'ERROR'
                    print 'Number of theta grid points is different in dusttemp_final.dat and theta.inp'
                    rfile.close()
                    return 5

            self.data = zeros([self.nx, self.ny, self.nz], dtype=float)
            dum = rfile.readline() 
            dum = rfile.readline() 
            if imirt==0:
                for ix in range(self.nx):
                    for iy in range(self.ny):
                        for iz in range(self.nz):
                            self.data[ix,iy,iz] = float(rfile.readline())
            else:
                for ix in range(self.nx):
                    for iy in range(self.ny/2):
                        for iz in range(self.nz):
                            self.data[ix,iy,iz] = float(rfile.readline())
                            self.data[ix,self.ny-1-iy,iz] = self.data[ix,iy,iz]

            rfile.close()
            self.importedFName = 'dusttemp_final.dat'
            self.tableFName = 'dusttemp_final.dat.ARTIST.tab'

        if varName=='dustdens':
            self.varName = 'dustdens'
            try:
                rfile = open('dustdens.inp', 'r')
            except:
                print 'ERROR'
                print 'dustdens.inp cannot be opened'
                return 6 

            dum = rfile.readline().split()
            ndust = int(dum[0])
            nr    = int(dum[1])
            nt    = int(dum[2])
            
            if int(dum[3])!=imirt:
                print 'ERROR'
                print 'Dust density is defined in a different theta grid than given in theta.inp'
                print 'Theta mirroring is different in dustdens.inp and theta.inp'
                rfile.close()
                return 3

            if ndust>1:
                print 'WARNING'
                print 'Multiple-dust species are not yet supported'
                print 'I take the temperature of the first dust species'

            if nr!=self.nx:
                print 'ERROR'
                print 'Number of radial grid points is different in dustdens.inp and radius.inp'
                rfile.close()
                return 4

            if imirt==1:
                if nt!=self.ny/2:
                    print 'ERROR'
                    print 'Number of theta grid points is different in dustdens.inp and theta.inp'
                    rfile.close()
                    return 5
            else:
                if nt!=self.ny:
                    print 'ERROR'
                    print 'Number of theta grid points is different in dustdens.inp and theta.inp'
                    rfile.close()
                    return 5

            self.data = zeros([self.nx, self.ny, self.nz], dtype=float)
            dum = rfile.readline() 
            if imirt==0:
                for ix in range(self.nx):
                    for iy in range(self.ny):
                        for iz in range(self.nz):
                            self.data[ix,iy,iz] = float(rfile.readline())* 1e3 
           
            else:
                for ix in range(self.nx):
                    for iy in range(self.ny/2):
                        for iz in range(self.nz):
                            self.data[ix,iy,iz] = float(rfile.readline())* 1e3 
                            self.data[ix,self.ny-1-iy,iz] = self.data[ix,iy,iz]
            
            self.importedFName = 'dustdens.inp.dat'
            self.tableFName = 'dustdens.inp.ARTIST.tab'

        if varName=='density':
            self.varName = 'density'
            try:
                rfile = open('dustdens.inp', 'r')
            except:
                print 'ERROR'
                print 'dustdens.inp cannot be opened'
                return 6

            dum = rfile.readline().split()
            ndust = int(dum[0])
            nr    = int(dum[1])
            nt    = int(dum[2])
            
            if int(dum[3])!=imirt:
                print 'ERROR'
                print 'Dust density is defined in a different theta grid than given in theta.inp'
                print 'Theta mirroring is different in dustdens.inp and theta.inp'
                rfile.close()
                return 3

            if ndust>1:
                print 'WARNING'
                print 'Multiple-dust species are present'
                print 'For the gas density I take their sum'

            if nr!=self.nx:
                print 'ERROR'
                print 'Number of radial grid points is different in dustdens.inp and radius.inp'
                rfile.close()
                return 4 

            if imirt==1:
                if nt!=self.ny/2:
                    print 'ERROR'
                    print 'Number of theta grid points is different in dustdens.inp and theta.inp'
                    rfile.close()
                    return 5
            else:
                if nt!=self.ny:
                    print 'ERROR'
                    print 'Number of theta grid points is different in dustdens.inp and theta.inp'
                    rfile.close()
                    return 5

            self.data = zeros([self.nx, self.ny, self.nz], dtype=float)
            dum = rfile.readline() 
            if imirt==0:
                for idust in range(ndust):
                    for ix in range(self.nx):
                        for iy in range(self.ny):
                            for iz in range(self.nz):
                                self.data[ix,iy,iz] = self.data[ix,iy,iz] + float(rfile.readline()) * 1e2 / (2.3*1.6726e-24) * 1e6 
                    dumstr = rfile.readline() 
            else:
                for idust in range(ndust):
                    for ix in range(self.nx):
                        for iy in range(self.ny/2):
                            for iz in range(self.nz):
                                self.data[ix,iy,iz] = self.data[ix,iy,iz] + float(rfile.readline()) * 1e2 / (2.3*1.6726e-24) * 1e6
                                self.data[ix,self.ny-1-iy,iz] = self.data[ix,iy,iz] 

                    dumstr = rfile.readline() 

            self.data = self.data.clip(1e-10, 1e99)
            self.importedFName = 'dustdens.inp.dat'
            self.tableFName = 'gasdens_from_dustdens.inp.ARTIST.tab'

    def _readData_radmc3d(self, varName=''):
        """
        Reads the data from RADMC-3D

        INPUT:
            varName - Name of the variable to be imported
                        for available variables see getVarNames("radmc3d")
        """

        self.cellCenter = False

        possibleVarNames = getVarNames('radmc3d')
        goAhead = False
        for ivar in possibleVarNames:
            if varName==ivar :
                goAhead=True
        if not goAhead:
            print 'ERROR'
            print 'Unknown variable name :'
            print varName
            print 'For the suppported variables use getVarNames("radmc3d")'
            return
        try :
            rfile = open('amr_grid.inp', 'r')
        except:
            print 'Error!' 
            print 'amr_grid.inp was not found!'
            return 1

        form       = float(rfile.readline())
        gridStyle  = float(rfile.readline())

        if gridStyle > 0 :
            print ' AMR and Delaunay grids are not yet supported'
            return
        else:
            self.gridStyle=gridStyle

        crd_system  = int(rfile.readline())
        if crd_system<100:
            self.gridType = 0
            cfact_x = 1e-2
            cfact_y = 1e-2
            cfact_z = 1e-2
        elif ((crd_system>=100)&(crd_system<200)):
            self.gridType = 2
            cfact_x = 1e-2
            cfact_y = 1e0
            cfact_z = 1e0
        elif ((crd_system>=200)&(crd_system<300)):
            self.gridType = 1
            cfact_x = 1e-2
            cfact_y = 1e0
            cfact_z = 1e-2
        else:
            rfile.close()
            print 'ERROR'
            print ' unsupported coordinate system in the amr_grid.inp file'
            print crd_system
            return 4

        grid_info   = float(rfile.readline())
        dum         = rfile.readline().split()
        self.act_dim = [int(dum[i]) for i in range(len(dum))]
        dum         = rfile.readline().split()
        self.nx,self.ny,self.nz    = int(dum[0]), int(dum[1]), int(dum[2])
        self.nxi,self.nyi,self.nzi = self.nx+1, self.ny+1, self.nz+1

        self.xi           = np.zeros(self.nxi, dtype=np.float64)
        self.yi           = np.zeros(self.nyi, dtype=np.float64)
        self.zi           = np.zeros(self.nzi, dtype=np.float64)
        
        for i in range(self.nxi): 
            self.xi[i] = float(rfile.readline()) * cfact_x
        for i in range(self.nyi): 
            self.yi[i] = float(rfile.readline()) * cfact_y
        for i in range(self.nzi): 
            self.zi[i] = float(rfile.readline()) * cfact_z

        self.x = (self.xi[0:self.nx] +  self.xi[1:self.nxi]) * 0.5
        self.y = (self.yi[0:self.ny] +  self.yi[1:self.nyi]) * 0.5
        self.z = (self.zi[0:self.nz] +  self.zi[1:self.nzi]) * 0.5

        rfile.close()
        self.varName = varName

        if varName=='density':
            
            try:
                rfile = open('dust_density.inp', 'r')
            except:
                print 'ERROR'
                print 'dust_density.inp cannot be opened'
                return 5

            dum = rfile.readline() 
            dum = rfile.readline() 
            if int(dum)!=(self.nx*self.ny*self.nz):
                print 'ERROR'
                print 'Number of gridpoints in dust_density.inp differ from that in amr_grid.inp'
                rfile.close()
                return 4 
            ndust = int(rfile.readline()) 
            
            self.data = np.zeros([self.nx, self.ny, self.nz], dtype=float)
            for iz in range(self.nz):
                for iy in range(self.ny):
                    for ix in range(self.nx):
                        self.data[ix,iy,iz] = float(rfile.readline()) * 1e2 / (2.3* 1.6726e-24) * 1e6

            rfile.close()
            self.importedFName = 'dust_density.inp'
            self.tableFName = 'gasdens_from_dust_density.inp.ARTIST.tab'


        if varName=='dustdens':
            try:
                rfile = open('dust_density.inp', 'r')
            except:
                print 'ERROR'
                print 'dust_density.inp cannot be opened'
                return 5

            dum = rfile.readline() 
            dum = rfile.readline() 
            if int(dum)!=(self.nx*self.ny*self.nz):
                print 'ERROR'
                print 'Number of gridpoints in dust_density.inp differ from that in amr_grid.inp'
                rfile.close()
                return 4
            ndust = int(rfile.readline()) 
            
            self.data = np.zeros([self.nx, self.ny, self.nz], dtype=float)
            for iz in range(self.nz):
                for iy in range(self.ny):
                    for ix in range(self.nx):
                        self.data[ix,iy,iz] = float(rfile.readline()) * 1e3 

            rfile.close()
            
            self.importedFName = 'dust_density.inp'
            self.tableFName = 'dust_density.inp.ARTIST.tab'



        if varName=='tdust':
            try:
                rfile = open('dust_temperature.dat', 'r')
            except:
                print 'ERROR'
                print 'dust_temperature.dat cannot be opened'
                return 5
            dum = rfile.readline()
            dum = rfile.readline() 
            if self.nx*self.ny*self.nz!=float(dum):
                print 'ERROR'
                print 'Number of gridpoints in dust_temperature.dat differ from that in amr_grid.inp'
                rfile.close()
                return 4
            
            ndust = int(rfile.readline()) 
            ndust = 1 

            self.data = np.zeros((self.nx, self.ny, self.nz), dtype=float)
            for idust in range(ndust):
                for iz in range(self.nz):
                    for iy in range(self.ny):
                        for ix in range(self.nx):
                            self.data[ix,iy,iz] = float(rfile.readline())

            self.importedFName = 'dust_temperature.inp'
            self.tableFName = 'dust_temperature.inp.ARTIST.tab'
        
        if varName=='temperature':
            try:
                rfile = open('gas_temperature.inp', 'r')
            except:
                print 'ERROR'
                print 'gas_temperature.dat cannot be opened'
                return 5
            
            dum = rfile.readline() 
            dum = rfile.readline() 
            if self.nx*self.ny*self.nz!=float(dum):
                print 'ERROR'
                print 'Number of gridpoints in gas_temperature.dat differ from that in amr_grid.inp'
                rfile.close()
                return 4
            

            self.data = np.zeros((self.nx, self.ny, self.nz), dtype=float)
            for iz in range(self.nz):
                for iy in range(self.ny):
                    for ix in range(self.nx):
                        self.data[ix,iy,iz] = float(rfile.readline())
            
            self.importedFName = 'gas_temperature.inp'
            self.tableFName = 'gas_temperature.inp.ARTIST.tab'
        
        if varName=='velocity':
            try:
                rfile = open('gas_velocity.inp', 'r')
            except:
                print 'ERROR'
                print 'gas_velocity.inp cannot be opened'
                return 5
            
            dum = rfile.readline() 
            dum = rfile.readline() 
            if self.nx*self.ny*self.nz!=float(dum):
                print 'ERROR'
                print 'Number of grid points in gas_velocity.inp differ from that in amr_grid.inp'
                rfile.close()
                return 4
            

            self.data = np.zeros((self.nx, self.ny, self.nz, 3), dtype=float)
            for iz in range(self.nz):
                for iy in range(self.ny):
                    for ix in range(self.nx):
                        vel = rfile.readline().split()
                        self.data[ix,iy,iz,0] = float(vel[0]) * 1e-2
                        self.data[ix,iy,iz,1] = float(vel[1]) * 1e-2
                        self.data[ix,iy,iz,2] = float(vel[2]) * 1e-2
            
            self.importedFName = 'gas_velocity.inp'
            self.tableFName = 'gas_velocity.inp.ARTIST.tab'
                        

         

            
         

            
