################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
from matplotlib.pylab import cm
from matplotlib import colors
import modellib
import ArtistTable 
import sys

def casa_colors():
    
    cdict = {'red': ((0.0, 0.0, 0.0),  
                     (0.2, 0.0888, 0.0888),
                     (0.4, 0.0888, 0.0888),
                     (0.5, 0., 0.),
                     (0.6, 0.9405, 0.9405),
                     (0.8, 1.0, 1.0),
                     (0.9, 1.0, 1.0),
                     (1.0, 1.0, 1.0)),
             'green': ((0.0, 0.0, 0.0),
                       (0.2, 0.3061, 0.3061),
                       (0.4, 0.7815, 0.7815),
                       (0.5, 1.0, 1.0),
                       (0.6, 0.9191, 0.9191),
                       (0.8, 0.0, 0.0),
                       (0.9, 0.3568, 0.3568),
                       (1.0, 1.0, 1.0)),
             'blue': ((0.0, 0.0, 0.0),
                      (0.2, 0.8259, 0.8259),
                       (0.4, 1.0, 1.0),
                       (0.5, 0., 0.),
                       (0.6, 0., 0.),
                       (0.8, 0., 0.),
                       (0.9, 1.0, 1.0),
                      (1.0, 1.0, 1.0))}
             
    colvel = colors.LinearSegmentedColormap('colvel',cdict,256)

    return colvel



def color_velocity():
    cdict = {'red': ((0.0, 1.0, 1.0),
                     (0.5, 0.0, 0.0),
                     (1.0, 0.0, 0.0)),
             'green': ((0.0, 0.0, 0.0),
                       (0.5, 1.0, 1.0),
                       (1.0, 0.0, 0.0)),
             'blue': ((0.0, 0.0, 0.0),
                      (0.5, 0.0, 0.0),
                      (1.0, 1.0, 1.0))}
             
    colvel = colors.LinearSegmentedColormap('colvel',cdict,256)

    return colvel


def blue1():
    cdict = {'red': ((0.0, 0.0, 0.0),
                     (0.6, 0.0, 0.0),
                     (1.0, 1.0, 1.0)),
             'green': ((0.0, 0.0, 0.0),
                       (0.6, 0.0, 0.0),
                       (1.0, 1.0, 1.0)),
             'blue': ((0.0, 0.0, 0.0),
                      (0.6, 1.0, 1.0),
                      (1.0, 1.0, 1.0))}
             
    mcm_blue1 = colors.LinearSegmentedColormap('mcm_blue1',cdict,256)

    return mcm_blue1

class GridColor():
    def __init__(self, parent=None, currentColor=None, currentColorMap=None, isCurrentSingleColor=False):

        self.colorMaps={'binary':cm.binary, 'Blue':blue1(), 'BrBG':cm.BrBG, 'RdBu':cm.RdBu, 'hot':cm.hot,\
                'hsv':cm.hsv, 'jet':cm.jet,'spectral':cm.spectral, 'color_velocity':color_velocity(), \
                'casa':casa_colors()}
        
        if currentColor:
            self.currentColor=currentColor
        else:
            self.currentColor='#FFFFFF'


        if currentColorMap:
            self.currentColorMap=currentColorMap
        else:
            self.currentColorMap='jet'

        self.currentColorMapID = self.colorMaps.keys().index(self.currentColorMap)

        if isCurrentSingleColor:
            self.isCurrentSingleColor=True
        else:
            self.isCurrentSingleColor=False

    def setCurrentColor(self, colorName=''):
        """
            Sets the current color to a single color with name 'colorName'
        """
        self.currentColor=colorName
        self.isCurrentSingleColor=True
    
    def setCurrentColorMap(self, colorMapName='', colorMapID=0):
        """
            Sets the current color to a color map with name 'colorMapName'
        """
        if colorMapName!='':
            self.currentColorMap=colorMapName
            self.isCurrentSigleColor=False
            self.currentColorMapID = self.colorMaps.keys().index(self.currentColorMap)
        else:
            self.currentColorMapID = colorMapID
            self.currentColorMap = self.colorMaps[self.colorMaps.keys()[colorMapID]]
            self.isCurrentSigleColor=False
        

class Grid():
    """
        Class for grid/mesh 
    """
    def __init__(self, parent=None, minval=0., maxval=0., nval=0., gridstyle='lin'):

        self.minval    = minval
        self.maxval    = maxval
        self.nval      = nval
        self.gridstyle = gridstyle
        self.val       = []
        if nval!=0:
            self.makegrid()

    def makegrid(self, minval=None, maxval=None, nval=None, gridstyle=None):
        """
            Creates a linear/logarithmic grid
        """
        if minval!=None:
            self.minval = minval

        if maxval!=None:
            self.maxval = maxval

        if nval!=None:
            self.nval = nval

        if gridstyle!=None:
            self.gridstyle = gridstyle

        if self.gridstyle=='lin':
            self.val = self.minval + (self.maxval - self.minval) * (np.arange(self.nval, dtype=np.float64) / (self.nval - 1e0))
        if self.gridstyle=='log':
            self.val = self.minval * (self.maxval / self.minval) ** (np.arange(self.nval, dtype=np.float64) / (self.nval - 1e0))

        

class AbundanceModel():
    """
    Model class for abundance 
    """
    def __init__(self):

        self.models = ('Drop')
        self.modelsCurrentIndex = 0

        self.regionSeparators = ('Temperature', 'Density', 'Radius')
        self.regionSeparatorsCurrentIndex = 2
        self.regionSeparatorConv = {'Temperature': [1.0], \
                                    'Density': [1e6, 1.0], \
                                    'Radius':[0.01,1.0,1.496e11,3.0857200e+16]}
        self.regionSeparatorUnits = {'Temperature': ['K'], \
                                    'Density': ['1/cm^3', '1/m^3'], \
                                    'Radius':['cm', 'm', 'AU','pc']}
        self.regionSeparatorUnitSelection = {'Temperature':'K', 'Density': '1/cm^3', 'Radius':'AU'}

        self.nRegion = 2
        self.regionBoundaries = [10.]
        self.regionAbundances = [1e-9, 1e-4]

        self.makeAbundanceTable(minScale=1.4e11, radius=1.4e11*2000.)

    def makeAbundanceTable(self, minScale=None, radius=None, writeTable=False):
       
        nr     = 300
        ntheta = 100
        nphi   = 36

        separator = self.regionSeparators[self.regionSeparatorsCurrentIndex]
        self.abundances = np.zeros([nr-1,ntheta-1,nphi-1], dtype=float)

        ri = minScale * (radius/minScale)**(np.arange(nr, dtype=float) / (float(nr)-1.))
        thetai = np.pi * np.arange(ntheta, dtype=float) / (float(ntheta)-1.)
        phii = 2.*np.pi * np.arange(nphi, dtype=float) / (float(nphi)-1.)
          
        r = np.sqrt(ri[1:] * ri[:-1])
        theta = 0.5 * (thetai[1:] + thetai[:-1])
        phi = 0.5 * (phii[1:] + phii[:-1])


        index = self.regionSeparatorUnits[separator].index(self.regionSeparatorUnitSelection[separator]) 
        conv  = self.regionSeparatorConv[separator][index]
        boundaries = [i*conv for i in self.regionBoundaries]

        if separator=='Temperature':
            for ir in range(nr-1):
                for it in range(ntheta-1):
                    x = r[ir] * np.sin(theta[it]) * np.cos(phi)
                    y =  r[ir] * np.sin(theta[it]) * np.sin(phi)
                    z = r[ir] * np.cos(theta[it])
           
                    var = [modellib.temperature(x[i], y[i], z) for i in range(nphi-1)]

                    self.abundances[ir,it,:] = self.regionAbundances[0]
                    if (self.nRegion>2):
                        for ireg in range(1,self.nRegion-1):
                            ii = ((var>boundaries[ireg-1])&(var<=boundaries[ireg]))
                            self.abundances[ir,ntheta,ii] = self.regionAbundances[ireg]

                    ii = (var > boundaries[self.nRegion-2])
                    self.abundances[ir,it,ii] = self.regionAbundances[self.nRegion-1]
                    
        if separator=='Density':
            for ir in range(nr-1):
                for it in range(ntheta-1):
                    x = r[ir] * np.sin(theta[it]) * np.cos(phi)
                    y =  r[ir] * np.sin(theta[it]) * np.sin(phi)
                    z = r[ir] * np.cos(theta[it])
            
                    var = [modellib.density(x[i], y[i], z) for i in range(nphi-1)]

                    self.abundances[ir,it,:] = self.regionAbundances[0]
                    if (self.nRegion>2):
                        for ireg in range(1,self.nRegion-1):
                            ii = ((var>boundaries[ireg-1])&(var<=boundaries[ireg]))
                            self.abundances[ir,ntheta,ii] = self.regionabundances[ireg]

                    ii = (var > boundaries[self.nRegion-2])
                    self.abundances[ir,it,ii] = self.regionAbundances[self.nRegion-1]
        
        if separator=='Radius':
           
            for ir in range(nr-1):
                var = r[ir]
               
                self.abundances[ir,:,:] = self.regionAbundances[0]
                if (self.nRegion>2):
                    for ireg in range(1,self.nRegion-1):
                        if (var>boundaries[ireg-1])&(var<=boundaries[ireg]):
                            self.abundances[ir,:,:] = self.regionAbundances[ireg]     


                if var>boundaries[self.nRegion-2]:
                    self.abundances[ir,:,:] = self.regionAbundances[self.nRegion-1]     

        
        if writeTable:
            table = ArtistTable.DataTable()

            table.varName = 'abundance'
            table.gridstyle=0
            table.gridType=2
            table.cellCenter=False
            table.nx = nr-1
            table.ny = ntheta-1
            table.nz = nphi-1
            table.nxi = nr
            table.nyi = ntheta
            table.nzi = nphi
            table.x   = r
            table.y   = theta
            table.z   = phi
            table.xi   = ri
            table.yi   = thetai
            table.zi   = phii
            table.act_dim = [1,1,1]
            table.data = np.array(self.abundances)

            table.writeTable()
