################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
import sys
from PyQt4 import QtCore, QtGui
from artist_gui.HeaderDisplayDialogBase import Ui_HeaderDisplayDialog
from artist_gui.ImageViewerMainWindowBase import Ui_ImageViewerMainWindow
from artist_gui.PixSpecMainWindowBase import Ui_mplFigurePixSpecMainWindow
from artist_gui.CustomizeDialogBase import *
from artist_gui.grid import *
from artist_gui.helper_functions import *
from artist_gui.griddialog import GridDialog
import pyfits
import copy as cp
import logging

try:
    from scipy import ndimage as ndi
    PVDEnabled = True
except:
    print ' WARNING '
    print ' SciPy module was not found, PV Diagrams cannot be calculated'
    PVDEnabled = False    

from matplotlib.pylab import *
log = logging.getLogger('artist')



class ImageData():
   
    _accepted_keywords = {'CTYPE':['RA--','GLON', 'ELON', 'DEC-', 'GLAT', 'ELAT', 'FREQ', 'LAMB', 'VELO', 'VRAD', 'STOK'],\
                         'CUNIT':['DEG', 'HZ', 'KHZ', 'MHZ', 'GHZ', 'MICRON', 'CM/S', 'M/S', 'KM/S'],\
                         'BTYPE':['INTENSITY', 'FLUX'],\
                         'BUNIT':['K', 'JY/BEAM', 'JY/PIXEL']}

    def __init__(self):
        self.imexists    = False
        self.imageFName  = ''
        self.header      = {}
        self.contImage   = False
        self.stokesImage = False
        self.image     = {'data':np.array(-1., dtype=float),\
                          'cdata':np.array(-1., dtype=float),\
                          'fdata':np.array(-1., dtype=float),\
                          'stokes_index':0,\
                          'stokes_axis' : 0,\
                          'ifreq':0,\
                          'crd_x':-1.,\
                          'crd_xi':-1.,\
                          'crd_y':-1.,\
                          'crd_yi':-1.,\
                          'crd_vel':-1.,\
                          'clipMin':-1.,\
                          'clipMax':-1.,\
                          'enableCLines':False,\
                          'logscale':False,\
                          'plotconv':False,\
                          'bmin':0.2,\
                          'bmaj':0.2,\
                          'pa':45.,\
                          'whatToPlotCLines':''}
        self.mmap      = {'data':np.array(-1., dtype=float),\
                          'cdata':np.array(-1., dtype=float),\
                          'nmom':0,\
                          'plotconv':False,\
                          'logscale':False,\
                          'mapchanged':False,\
                          'enableCLines':False,\
                          'whatToPlotCLines':'',\
                          'crd_x':-1.,\
                          'crd_xi':-1.,\
                          'crd_y':-1.,\
                          'crd_yi':-1.,\
                          'crd_vel':-1.,\
                          'threshold':np.array([1e-10, 1e30], dtype=float),\
                          'channel_ind':np.array([0,1000], dtype=np.int),\
                          'clipMin':np.array([-1., -1., -1.], dtype=float),\
                          'clipMax':np.array([-1., -1., -1.], dtype=float),\
                          'bmin':0.2,\
                          'bmaj':0.2,\
                          'pa':45.}
        self.pvdiag     = {'data':np.array(-1., dtype=float),\
                          'cdata':np.array(-1., dtype=float),\
                          'fdata':np.array(-1., dtype=float),\
                          'posang':0.,\
                          'posind':0,\
                          'stokes_index':0,\
                          'stokes_axis' : 0,\
                          'ifreq':0,\
                          'crd_x':-1.,\
                          'crd_xi':-1.,\
                          'crd_y':-1.,\
                          'crd_yi':-1.,\
                          'crd_vel':-1.,\
                          'clipMin':-1.,\
                          'clipMax':-1.,\
                          'logscale':False,\
                          'plotconv':False,\
                          'bmin':0.2,\
                          'bmaj':0.2,\
                          'pa':45.,\
                          'whatToPlotCLines':''}


    def get_psf(self):
        """
        Function to generate a two dimensional Gaussian PSF
    
        SYNTAX:
           result = get_psf(nx=300,ny=300,fwhm=[0.1, 0.2], pa=15.0, pscale=[0.1, 0.2])

        INPUT:
             nx      : image size in the first dimension
             ny      : image size in the second dimension
             fwhm    : full width at half maximum of the psf in each dimension [fwhm_x, fwhm_y]
             pa      : position angle of the gaussian if the gaussian is not symmetric
             pscale  : pixelscale of the image, if set fwhm should be in the same unit, if not set unit of fwhm is pixels

        OUTPUT:
            result  : dictionary containing the following keys
            'psf'   : two dimensional numpy array containing the normalized psf
            'x'     : first coordinate axis of the psf
            'y'     : seonc coordinate axis of the psf
          
        """

        ny,nx = self.image['data'].shape[::-1][0:2]
        pscale=[3600.*float(self.header['CDELT1']), 3600.*float(self.header['CDELT2'])] 
        
        fwhm = [self.image['bmaj'], self.image['bmin']]
        pa = self.image['pa'] + 90.
        

        if (pscale!=None):
            dx,dy = pscale[0], pscale[1]
        else:
            dx,dy = 1., 1.

        x = ((np.arange(nx, dtype=float) + 0.5) - nx/2) * dx
        y = ((np.arange(ny, dtype=float) + 0.5) - ny/2) * dy

        sigmax = fwhm[0] / (2.0 * np.sqrt(2.0 * np.log(2.)))
        sigmay = fwhm[1] / (2.0 * np.sqrt(2.0 * np.log(2.)))
        norm   = 1./(2. * np.pi * sigmax * sigmay)



        sin_pa = np.sin(pa/180.*np.pi - np.pi/2.)
        cos_pa = np.cos(pa/180.*np.pi - np.pi/2.)

        psf = np.zeros([nx,ny], dtype=float)
        norm = 1./(sigmax * sigmay * 2. * np.pi) 
        for ix in range(nx):
            for iy in range(ny):
                xx = cos_pa * x[ix] - sin_pa * y[iy]
                yy = sin_pa * x[ix] + cos_pa * y[iy]

                psf[ix,iy] = norm * np.exp(-0.5*xx*xx/sigmax/sigmax - 0.5*yy*yy/sigmay/sigmay)

        psf = psf / psf.sum()

        res = {'psf':psf, 'x':x, 'y':y}

        return res

    def imconv(self):

        print 'Image convolution'

        dum = self.get_psf()
        psf = dum['psf']/dum['psf'].sum(0).sum(0)

        f_psf = np.fft.fft2(psf)

        self.image['cdata'] = np.zeros(self.image['data'].shape, dtype=float)
        for ifreq in range(self.image['data'].shape[1]):
            f_imag  = np.fft.fft2(self.image['data'][self.image['stokes_index'],ifreq,:,:])
            f_cimag = f_psf * f_imag
            self.image['cdata'][self.image['stokes_index'],ifreq,:,:] = abs(np.fft.ifftshift(np.fft.ifft2(f_cimag)))
            
    def makeMomentMap(self, conv=False):

         print 'Make moment map'

         id1 = self.mmap['channel_ind'][0]
         if id1<0: id1=0
         id2 = self.mmap['channel_ind'][1]
         if not conv:
             if id2>self.image['data'].shape[1]:id2 = self.image['data'].shape[1]
             imag = np.array(self.image['data'][self.image['stokes_index'],id1:id2,:,:])

         else:
             if id2>self.image['cdata'].shape[1]:id2 = self.image['cdata'].shape[1]
             imag = np.array(self.image['cdata'][self.image['stokes_index'],id1:id2,:,:])

         imag[imag<self.mmap['threshold'][0]] = 0.
         imag[imag>self.mmap['threshold'][1]] = 0.
        
         nvel, nx, ny = imag.shape
         nvel = self.image['data'].shape[1]
         mmap = np.zeros([nx,ny,3], dtype=np.float64)

         ivel           = np.zeros(nvel+1, dtype=np.float64)
         ivel[1:nvel]   = (self.image['crd_vel'][1:] + self.image['crd_vel'][:-1]) * 0.5
         ivel[0]        = ivel[1] + (ivel[1] - ivel[2])
         ivel[nvel]     = ivel[nvel-1] + (ivel[nvel-1] - ivel[nvel-2])
         dv             = (ivel[1:] - ivel[:-1])[id1:id2]

         nvel = imag.shape[0]

         if dv[0]<0: dv = -dv
        

         w = np.zeros(imag.shape, dtype=np.float64)
         dvel = np.zeros(imag.shape, dtype=np.float64)
         vel = self.mmap['crd_vel'][id1:id2]
         
         for inu in range(imag.shape[0]):
             dvel[inu,:,:] = dv[inu]
            
         mmap[:,:,0] = (imag*dvel).sum(0)

         norm = np.array(mmap[:,:,0])
         norm[norm==0.0]=1.0
         
         for inu in range(imag.shape[0]):
             w[inu,:,:] = vel[inu]
         mmap[:,:,1] = (imag*dvel*w).sum(0) / norm


         for inu in range(imag.shape[0]):
             w[inu,:,:] = vel[inu]-mmap[:,:,1]
         
         dum = (imag*dvel)*w*w
         mmap[:,:,2] = dum.sum(0) / norm
         

         if not conv:
             self.mmap['data'] = mmap
         else:
             self.mmap['cdata'] = mmap

         if not conv:
             for i in range(3):
                 self.mmap['clipMin'][i] = self.mmap['data'][:,:,i].min()
                 self.mmap['clipMax'][i] = self.mmap['data'][:,:,i].max()
         else:
             for i in range(3):
                 self.mmap['clipMin'][i] = self.mmap['cdata'][:,:,i].min()
                 self.mmap['clipMax'][i] = self.mmap['cdata'][:,:,i].max()


    def makePVDiagram(self,conv=False):

        if not PVDEnabled: return

        if not conv:
            if len(self.image['data'].shape)==4:
                dum = ndi.rotate(self.image['data'][0,0,:,:], self.pvdiag['posang'])
                self.pvdiag['data'] = np.zeros([self.image['data'].shape[1], dum.shape[0], dum.shape[1]], dtype=np.float)
                for ivel in range(self.image['crd_vel'].shape[0]):
                    self.pvdiag['data'][ivel,:,:] = ndi.rotate(self.image['data'][:,::-1,:,::-1][0,ivel,:,:], self.pvdiag['posang'])

            elif len(self.image['data'].shape)==3:
                dum = ndi.rotate(self.image['data'][0,:,:], self.pvdiag['posang'])
                self.pvdiag['data'] = np.zeros([self.image['data'].shape[0], dum.shape[0], dum.shape[1]], dtype=np.float)
                for ivel in range(self.image['crd_vel'].shape[0]):
                    self.pvdiag['data'][ivel,:,:] = ndi.rotate(self.image['data'][::-1,:,::-1][ivel,:,:], self.pvdiag['posang'])

            if (self.pvdiag['crd_vel'][1]<self.pvdiag['crd_vel'][0]):
                self.pvdiag['data'] = self.pvdiag['data'][::-1,:,:]

        else:
            if len(self.image['cdata'].shape)==4:
                dum = ndi.rotate(self.image['cdata'][0,0,:,:], self.pvdiag['posang'])
                self.pvdiag['cdata'] = np.zeros([self.image['cdata'].shape[1], dum.shape[0], dum.shape[1]], dtype=np.float)
                for ivel in range(self.image['crd_vel'].shape[0]):
                    self.pvdiag['cdata'][ivel,:,:] = ndi.rotate(self.image['cdata'][:,::-1,:,::-1][0,ivel,:,:], self.pvdiag['posang'])

            elif len(self.image['cdata'].shape)==3:
                dum = ndi.rotate(self.image['cdata'][0,:,:], self.pvdiag['posang'])
                self.pvdiag['cdata'] = np.zeros([self.image['cdata'].shape[0], dum.shape[0], dum.shape[1]], dtype=np.float)
                for ivel in range(self.image['crd_vel'].shape[0]):
                    self.pvdiag['cdata'][ivel,:,:] = ndi.rotate(self.image['cdata'][::-1,:,::-1][ivel,:,:], self.pvdiag['posang'])
        
            if (self.pvdiag['crd_vel'][1]<self.pvdiag['crd_vel'][0]):
                self.pvdiag['cdata'] = self.pvdiag['cdata'][::-1,:,:]

    def makePVDiagCrdaxes(self):

        self.pvdiag['crd_vel'] = self.image['crd_vel']

        nix = self.image['crd_x'].shape[0]
        niy = self.image['crd_y'].shape[0]

        ixc = (self.image['crd_x'].max() + self.image['crd_x'].min()) * 0.5
        iyc = (self.image['crd_y'].max() + self.image['crd_y'].min()) * 0.5 
        xx = self.image['crd_x'] - ixc
        yy = self.image['crd_y'] - iyc

        x0, y0 = xx.max(), yy.max()
        r = np.sqrt(x0*x0 + y0*y0)
        phi = np.arccos(x0 / (r+1e-90))

        x1  = r * np.cos(self.pvdiag['posang']/180.*np.pi)
        y1  = r * np.sin(self.pvdiag['posang']/180.*np.pi)
        x1r = x1 * np.cos(phi) - y1*np.sin(phi)
        y1r = x1 * np.sin(phi) + y1*np.cos(phi)
        nx = self.pvdiag['data'].shape[1]
        ny = self.pvdiag['data'].shape[2]
        self.pvdiag['crd_x'] = (np.arange(nx, dtype=float) / (float(nx)-1.) - 0.5) * 2. * x1r + ixc
        self.pvdiag['crd_y'] = (np.arange(ny, dtype=float) / (float(ny)-1.) - 0.5) * 2. * y1r + iyc


    def _isHeaderCompatible(self, header):

        if ((header['NAXIS']>4)|(header['NAXIS']<2)):
            msg = "Image cannot be opened due to header keyword mismatch (NAXIS values of 2,3 or 4 are allowed)"
            reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
            log.exception(msg)
            return False

        hdr_keys = [i.upper() for i in header.keys()]
        for iax in range(header['NAXIS']):
            spectral_axis = False

            ciax_txt = 'CTYPE'+str(iax+1).strip()
            keywordOK = False

            for key in self._accepted_keywords['CTYPE']:
                if header[ciax_txt].strip()[0:4].upper()==key: keywordOK = True
            if not keywordOK: 
                msg = "Image cannot be opened due to unknown header keyword ("+ciax_txt+"="+header[ciax_txt]+")"
                reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
                log.exception(msg)
                return False
            

            if header[ciax_txt].strip()[0:4]!='STOK':
                ciax_txt = 'CUNIT'+str(iax+1).strip()
                keywordOK = False
                
                if (iax+1)<=2:
                    if not header.__contains__(ciax_txt):
                        dum = 'CTYPE'+str(iax+1).strip()
                        axis_spatial=False
                        if header[dum].strip()[0:4]=='RA--': axis_spatial=True
                        if header[dum].strip()[0:4]=='DEC-': axis_spatial=True
                        if header[dum].strip()[0:4]=='GLON': axis_spatial=True
                        if header[dum].strip()[0:4]=='GLAT': axis_spatial=True
                        if header[dum].strip()[0:4]=='ELON': axis_spatial=True
                        if header[dum].strip()[0:4]=='ELAT': axis_spatial=True
                        
                        if not axis_spatial:
                            msg = "Image cannot be opened due to unknown header keyword ("+ciax_txt+"="+header[ciax_txt]+")"
                            reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
                            log.exception(msg)
                            return False
                        else:
                            keywordOK = True
                    else:
                        for key in self._accepted_keywords['CUNIT']:
                            if header[ciax_txt].strip().upper()==key: keywordOK = True
                        if not keywordOK: 
                            msg = "Image cannot be opened due to unknown header keyword ("+ciax_txt+"="+header[ciax_txt]+")"
                            reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
                            log.exception(msg)
                            return False
                else:                
                    for key in self._accepted_keywords['CUNIT']:
                        if header[ciax_txt].strip().upper()==key: keywordOK = True
                    if not keywordOK: 
                        msg = "Image cannot be opened due to unknown header keyword ("+ciax_txt+"="+header[ciax_txt]+")"
                        reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
                        log.exception(msg)
                        return False

        if hdr_keys.__contains__('BTYPE'):
            keywordOK = False
            for key in self._accepted_keywords['BTYPE']:
                if header['BTYPE'].strip().upper()==key: keywordOK = True
            if not keywordOK: 
                msg = "Image cannot be opened due to unknown header keyword (BTYPE="+header['BTYPE']+")"
                reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
                log.exception(msg)
                return False

        if not hdr_keys.__contains__('BUNIT'):
                msg = "Fits header does not contain the keyword BUNIT, thus BUNIT='PIXEL VALUE' will be used"
                print msg 
                header.update('BUNIT', 'VALUE', '')
                log.exception(msg)

        return True

    def writeImage(self, fname=''):
        """
        """
       
        v      = self.pvdiag['crd_vel'] 
        if self.pvdiag['crd_y'].max()>self.pvdiag['crd_x'].max():
            x = self.pvdiag['crd_y']
            y = self.pvdiag['crd_x']
        else:
            x = self.pvdiag['crd_x']
            y = self.pvdiag['crd_y']


        if len(self.pvdiag['data'].shape)==3:
            hdu     = pyfits.PrimaryHDU(self.pvdiag['data'].swapaxes(0,1)[:,::-1,::-1])
        elif len(self.pvdiag['data'].shape)==4:
            hdu     = pyfits.PrimaryHDU(self.pvdiag['data'].swapaxes(0,3).swapaxes(1,2))

        hdulist = pyfits.HDUList([hdu])
        
        hdulist[0].header.update('CRPIX2', 1.0, ' ')
        hdulist[0].header.update('CDELT2', (v[0]-v[1]), '')
        hdulist[0].header.update('CRVAL2', v[v.shape[0]-1], '')
        hdulist[0].header.update('CUNIT2', 'KM/S    ', '')
        hdulist[0].header.update('CTYPE2', 'VELO    ', '')
       
        hdulist[0].header.update('CRPIX1', 1.0, '')
        hdulist[0].header.update('CDELT1', (x[0]-x[1]), '')
        hdulist[0].header.update('CRVAL1', x[x.shape[0]-1], '')
        hdulist[0].header.update('CUNIT1', 'ARCSEC', '')
        hdulist[0].header.update('CTYPE1', 'X', '')
        
        hdulist[0].header.update('CRPIX3', 1.0, '')
        hdulist[0].header.update('CDELT3', (y[1]-y[0]), '')
        hdulist[0].header.update('CRVAL3', y[0], '')
        hdulist[0].header.update('CUNIT3', 'ARCSEC', '')
        hdulist[0].header.update('CTYPE3', 'Y', '')
        
        hdulist[0].header.update('BUNIT', self.header['BUNIT'], '')
        hdulist[0].header.update('BZERO', self.header['BZERO'], '')
        hdulist[0].header.update('BSCALE', self.header['BSCALE'], '')
        
        #hdulist[0].header.update('EPOCH', self.header['EPOCH'], '')

        if os.path.exists(fname):
            os.remove(fname)
            hdu.writeto(fname)
        else:
            hdu.writeto(fname)

    def readImage(self):

        log.debug('Reading image')
        try:
            data     = pyfits.getdata(self.imageFName)
            header   = pyfits.getheader(self.imageFName)

        except:
            log.exception('Image '+self.imageFName+' cannot be opened')
            reply = QtGui.QMessageBox.critical(self, "Error", 'Cannot open FITS file',  QtGui.QMessageBox.Ok)
            self.imexists = False
            return


        if not self._isHeaderCompatible(header):
            sys.exit()

        axis_id = []
        nx = -1
        ny = -1
        nv = -1
        ns = -1
        for iax in range(header['NAXIS']):
            iax_str = str(iax+1).strip()
            spec_axis = ''

            axis_type = header['CTYPE'+iax_str].strip().upper()[0:4]
            if axis_type=='STOK': axis_id.append(0)
            if axis_type=='FREQ': axis_id.append(1)
            if axis_type=='LAMB': axis_id.append(1)
            if axis_type=='VELO': axis_id.append(1)
            if axis_type=='VRAD': axis_id.append(1)
            if axis_type=='DEC-': axis_id.append(2)
            if axis_type=='GLAT': axis_id.append(2)
            if axis_type=='ELAT': axis_id.append(2)
            if axis_type=='RA--': axis_id.append(3)
            if axis_type=='GLON': axis_id.append(3)
            if axis_type=='ELON': axis_id.append(3)
           

            if axis_id[iax]==3:
                nx                  = header['NAXIS'+iax_str]
                self.image['crd_x'] = (((np.arange(nx, dtype=float)+1)-header['CRPIX'+iax_str])*header['CDELT'+iax_str]*3600.)

            if axis_id[iax]==2:
                ny                  = header['NAXIS'+iax_str]
                self.image['crd_y'] = (((np.arange(ny, dtype=float)+1)-header['CRPIX'+iax_str])*header['CDELT'+iax_str]*3600.)

            if axis_id[iax]==1:
                nv                  = header['NAXIS'+iax_str]
                if nv==1: self.contImage==True
                dum = (((np.arange(header['NAXIS'+iax_str], dtype=float)+1)-header['CRPIX'+iax_str])*header['CDELT'+iax_str] + header['CRVAL'+iax_str])
                if ((axis_type!='VELO')|(axis_type!='VRAD')):
                    if not header.keys().__contains__('RESTFREQ'):
                        if not header.keys().__contains__('RESTFRQ'):
                            reply = QtGui.QMessageBox.warning(None,"WARNING", "According to the fits header the spectral axis is frequency or wavelength but"+
                                    " no reference frequency (RESTFREQ, RESTFRQ keywords) is given. The central frequency in the frequency axis is taken "+
                                    " to be the rest frequency." , QtGui.QMessageBox.Ok)

                            log.warning('Image '+self.imageData.imageFName+' is opened but no reference frequency is given. The central channel taken to be the reference')
                            restfreq = np.median(dum)
                            self.imexists = True
                        else:
                            restfreq = header['RESTFRQ']
                    else:
                        restfreq = header['RESTFREQ']
                

                if header['CTYPE'+iax_str].strip()[0:4].upper()=='FREQ':
                    if header['CUNIT'+iax_str].strip().upper()=='HZ':
                        conv = 1.0
                    if header['CUNIT'+iax_str].strip().upper()=='KHZ':
                        conv = 1e3
                    if header['CUNIT'+iax_str].strip().upper()=='MHz':
                        conv = 1e6
                    if header['CUNIT'+iax_str].strip().upper()=='GHz':
                        conv = 1e9

                    dum = dum * conv
                    self.image['crd_vel'] = 299792458.0 * (1.-dum/restfreq) 


                elif header['CTYPE'+iax_str].strip()[0:4].upper()=='LAMB':
                    if header['CUNIT'+iax_str].strip().upper()=='MICRON':
                        conv = 1e-6
                    dum = 299792458.0/dum/conv
                    self.image['crd_vel'] = 299792458.0 * (1.-dum/restfreq) 

                elif header['CTYPE'+iax_str].strip()[0:4].upper()=='VELO':
                    if header['CUNIT'+iax_str].strip().upper()=='CM/S':
                        conv = 1e-2
                    elif header['CUNIT'+iax_str].strip().upper()=='M/S':
                        conv = 1.0
                    elif header['CUNIT'+iax_str].strip().upper()=='KM/S':
                        conv = 1e3
                    self.image['crd_vel'] = dum * conv


                elif header['CTYPE'+iax_str].strip()[0:4].upper()=='VRAD':
                    if header['CUNIT'+iax_str].strip().upper()=='CM/S':
                        conv = 1e-2
                    elif header['CUNIT'+iax_str].strip().upper()=='M/S':
                        conv = 1.0
                    elif header['CUNIT'+iax_str].strip().upper()=='KM/S':
                        conv = 1e3
                    self.image['crd_vel'] = dum * conv


                self.image['crd_vel'] = self.image['crd_vel']*1e-3

            if axis_id[iax]==0: 
                ns               = header['NAXIS'+iax_str]
                self.stokesImage = True

        
        if len(axis_id)==3:
            if not axis_id.__contains__(0):
                axis_id.append(0)
            if not axis_id.__contains__(1):
                axis_id.append(1)
                self.image['crd_vel'] = np.array([0.])

            s = data.shape
            self.image['fdata'] = np.zeros([1,s[0], s[1], s[2]], dtype=np.float)
            self.image['fdata'][0,:,:,:] = data


        elif len(axis_id)==4:
            self.image['fdata'] = np.array(data)
            
        

        if axis_id[0]!=3:
           ind1 = 0
           ind2 = axis_id.index(3)
           self.image['fdata'] = self.image['fdata'].swapaxes(3-ind1,3-ind2)
           dd = axis_id[ind1]
           axis_id[ind1] = axis_id[ind2]
           axis_id[ind2] = dd
        
        if axis_id[1]!=2:
           ind1 = 1
           ind2 = axis_id.index(2)
           self.image['fdata'] = self.image['fdata'].swapaxes(3-ind1,3-ind2)
           dd = axis_id[ind1]
           axis_id[ind1] = axis_id[ind2]
           axis_id[ind2] = dd
 
        if axis_id[2]!=1:
           ind1 = 2
           ind2 = axis_id.index(1)
           self.image['fdata'] = self.image['fdata'].swapaxes(3-ind1,3-ind2)
           dd = axis_id[ind1]
           axis_id[ind1] = axis_id[ind2]
           axis_id[ind2] = dd


        self.image['data']=np.array(self.image['fdata'])
        self.imexists = True
        
                
        if data.min()>0.:
            self.image['clipMin'] = data.min()
        else:
            self.image['clipMin'] = 0.

        self.image['clipMax'] = data.max()
           
        self.header = header


        self.image['crd_xi'] = np.zeros(nx+1, dtype=np.float64)
        dum = (self.image['crd_x'][1:nx] + self.image['crd_x'][0:nx-1]) * 0.5
        self.image['crd_xi'][1:nx] = dum
        self.image['crd_xi'][0] = dum[0] + dum[0]-dum[1]
        self.image['crd_xi'][nx] = dum[nx-2] + dum[nx-2] - dum[nx-3]

        self.image['crd_yi'] = np.zeros(ny+1, dtype=np.float64)
        dum = (self.image['crd_y'][1:ny] + self.image['crd_y'][0:ny-1]) * 0.5
        self.image['crd_yi'][1:ny] = dum
        self.image['crd_yi'][0] = dum[0] + dum[0]-dum[1]
        self.image['crd_yi'][ny] = dum[ny-2] + dum[ny-2] - dum[ny-3]
       
        self.mmap['cdelt1']  = str2float(header['CDELT1'], 'Header keyword CDELT1')
        self.mmap['cdelt2']  = str2float(header['CDELT2'], 'Header keyword CDELT2')
        self.mmap['crd_x']   = np.array(self.image['crd_x'])
        self.mmap['crd_y']   = np.array(self.image['crd_y'])
        self.mmap['crd_xi']  = np.array(self.image['crd_xi'])
        self.mmap['crd_yi']  = np.array(self.image['crd_yi'])
        self.mmap['crd_vel'] = np.array(self.image['crd_vel'])

        self.pvdiag['cdelt1']  = str2float(header['CDELT1'], 'Header keyword CDELT1')
        self.pvdiag['cdelt2']  = str2float(header['CDELT2'], 'Header keyword CDELT2')
        self.pvdiag['crd_x']   = np.array(self.image['crd_x'])
        self.pvdiag['crd_y']   = np.array(self.image['crd_y'])
        self.pvdiag['crd_xi']  = np.array(self.image['crd_xi'])
        self.pvdiag['crd_yi']  = np.array(self.image['crd_yi'])
        self.pvdiag['crd_vel'] = np.array(self.image['crd_vel'])
        self.pvdiag['clipMin'] = self.image['clipMin']
        self.pvdiag['clipMax'] = self.image['clipMax']

        log.debug('Image is read')
        


        return 

class mplFigurePixSpecMainWindow(QtGui.QMainWindow, Ui_mplFigurePixSpecMainWindow):

    def __init__(self, parent = None):
        super(mplFigurePixSpecMainWindow, self).__init__(parent)
        self.setupUi(self)

        self.connect(self.PixSpecCloseButton,QtCore.SIGNAL("clicked()"), self.closeWindow)

    def closeWindow(self):
        self.setVisible(False)

class HeaderDialog(QtGui.QDialog, Ui_HeaderDisplayDialog):

    def __init__(self, parent = None):
        super(HeaderDialog, self).__init__(parent)
        self.setupUi(self)

        self.connect(self.CloseButton,QtCore.SIGNAL("clicked()"), self.accept)
        self.HDRTextEdit.setReadOnly(True)

    def clear(self):

        self.HDRTextEdit.clear()
        
    def fillupHeader(self, hdr):

        ascl = hdr.ascardlist()

        for key in ascl.keys():
            if key.strip()!='HISTORY':
                val = ascl[key].value
                try: 
                    val_txt = ('%.12e'%val)
                except:
                    val_txt = str('%-18s'%val)
                txt = ("%-8s"%key) + ' = ' + val_txt + ' / ' + ascl[key].comment + '\n'
                self.HDRTextEdit.insertPlainText(txt)
       
        hist = hdr.get_history()
        if int(pyfits.__version__[0])<3:
            for i in range(len(hist)):
                txt = 'HISTORY ' + hist[i]+ '\n'
                self.HDRTextEdit.insertPlainText(txt)
        else:
            for i in range(len(hist)):
                txt = 'HISTORY ' + hist[i].value + '\n'
                self.HDRTextEdit.insertPlainText(txt)


class ImageViewerCustomizeDialog(QtGui.QDialog, Ui_CustomizeDialog):

    
    updateCustomization = QtCore.pyqtSignal(object,int,name='updateCustomization')
    writeCustomization = QtCore.pyqtSignal(object,name='writeCustomization')


    def __init__(self, parent=None, configParams=None):
        super(ImageViewerCustomizeDialog, self).__init__(parent)
        self.setupUi(self)

        self.configParams = cp.deepcopy(configParams)
        self.setStyleSheet('font-family: '+self.configParams.windowParams['font_family']+\
                '; font-size: '+self.configParams.windowParams['font_size']+'pt;')
        
        self.ARTISTFrame.setEnabled(False)
        self.ARTISTFrame.setVisible(False)
        self.setParamValues()

        self.connect(self.PrefDoneButton, QtCore.SIGNAL("clicked()"), self.readOutAllPrefsAndExit)
        self.connect(self.PrefSetAsDefaultButton, QtCore.SIGNAL("clicked()"), self.writeDefaultsToFile)
        self.connect(self.PrefCancelButton, QtCore.SIGNAL("clicked()"), self.exitWithoutReadOut)
        self.connect(self.PrefParaViewBrowseButton, QtCore.SIGNAL("clicked()"), self.openParaViewBrowser)
        self.connect(self.PrefWindowFontSelectorButton, QtCore.SIGNAL("clicked()"), self.openFontDialog)
        self.connect(self.PrefCanvasBgColorSelectorButton, QtCore.SIGNAL("clicked()"), self.browseCanvasBgColor)
        self.connect(self.PrefCanvasFgColorSelectorButton, QtCore.SIGNAL("clicked()"), self.browseCanvasFgColor)

    def setParamValues(self):


        self.PrefWindowFontFamilyLedit.setText(self.configParams.windowParams['font_family'])
        self.PrefWindowFontSizeLedit.setText(str(self.configParams.windowParams['font_size']).strip())

        self.PrefCanvasFontSizeLedit.setText(str(self.configParams.figureParams['font_size']).strip())

        color = QtGui.QColor(0,0,0)
        color.setNamedColor(self.configParams.figureParams['bgcolor'])
        self.PrefCanvasBgColorLedit.setPalette(QtGui.QPalette(color))
        self.PrefCanvasBgColorLedit.setAutoFillBackground(True)

        color = QtGui.QColor(0,0,0)
        color.setNamedColor(self.configParams.figureParams['fgcolor'])
        self.PrefCanvasFgColorLedit.setPalette(QtGui.QPalette(color))
        self.PrefCanvasFgColorLedit.setAutoFillBackground(True)


    def openFontDialog(self):
        
        oldFont = self.parent().font()
        dum = QtGui.QFontDialog()
        dum.setCurrentFont(oldFont)
        dum.exec_()
        if dum.result()==1:
            newFont = dum.selectedFont()
            family = str(newFont.family())
            psize = str(newFont.pointSize()).strip()
           
            self.setStyleSheet('font-family: '+family+'; font-size: '+psize+'pt;')
            self.configParams.windowParams['font_family'] = family
            self.configParams.windowParams['font_size'] = psize
            self.PrefWindowFontFamilyLedit.setText(family) 
            self.PrefWindowFontSizeLedit.setText(psize) 

    def browseCanvasFgColor(self):
        color = QtGui.QColorDialog.getColor(QtCore.Qt.green, self)

        if color.isValid(): 
            self.PrefCanvasFgColorLedit.setPalette(QtGui.QPalette(color))
            self.PrefCanvasFgColorLedit.setAutoFillBackground(True)
            self.configParams.figureParams['fgcolor']=str(color.name())


    def browseCanvasBgColor(self):
        color = QtGui.QColorDialog.getColor(QtCore.Qt.green, self)

        if color.isValid(): 
            self.PrefCanvasBgColorLedit.setPalette(QtGui.QPalette(color))
            self.PrefCanvasBgColorLedit.setAutoFillBackground(True)
            self.configParams.figureParams['bgcolor']=str(color.name())

    def openParaViewBrowser(self):
        
        if sys.platform=='darwin':
            dum = str(QtGui.QFileDialog.getExistingDirectory(self, 'Open Paraview.app',  options=QtGui.QFileDialog.DontUseNativeDialog))
            
            sdum = dum.split()
            ParaviewDir = sdum[0]
            for i in range(1,len(sdum)):
                ParaviewDir = ParaviewDir + '\ ' + sdum[i]

            command = sp.Popen(['find '+ParaviewDir+' -name paraview'], shell=True, stdout=sp.PIPE).communicate()[0].split('\n')
          
            for i in range(len(command)):
                if os.path.isfile(command[i]):
                    com = command[i]
                    break
                else:
                    ParaviewDir = ''

            if dum!='':
                self.configParams.paraViewCommand = com
                self.PrefParaViewComLedit.setText(com)
                python_ver = str(sys.version_info[0])+'.'+str(sys.version_info[1])
                dirname = os.environ['ARTIST_ROOT']+'/lib/python'+python_ver+'/site-packages/artist_gui/'
                wfile = open(dirname+'/paraview.com','w')
                wfile.write(com)
                wfile.close
            else:
                reply = QtGui.QMessageBox.critical(self, "Error",
                        'The paraview command was not found in '+dum,
                        QtGui.QMessageBox.Ok)
        else:
            com = str(QtGui.QFileDialog.getOpenFileName(self, 'Open Paraview',  options=QtGui.QFileDialog.DontUseNativeDialog))

            if com!='':
                self.configParams.paraViewCommand = com
                self.PrefParaViewComLedit.setText(com)
                python_ver = str(sys.version_info[0])+'.'+str(sys.version_info[1])
                dirname = os.environ['ARTIST_ROOT']+'/lib/python'+python_ver+'/site-packages/artist_gui/'
                wfile = open(dirname+'/paraview.com','w')
                wfile.write(com)
                wfile.close
            else:
                reply = QtGui.QMessageBox.critical(self, "Error",
                        'The paraview command was not found in '+dum,
                        QtGui.QMessageBox.Ok)


       
    def exitWithoutReadOut(self):   

        self.reject()

    def writeDefaultsToFile(self):
        self.configParams.windowParams['font_family'] = self.PrefWindowFontFamilyLedit.text()
        self.configParams.windowParams['font_size'] = str(str2int(self.PrefWindowFontSizeLedit.text(), 'Font Size (Window)'))
        self.configParams.figureParams['font_size'] = str(str2int(self.PrefCanvasFontSizeLedit.text(), 'Font Size (Image Canvas)'))
        self.configParams.unitSystem = str(self.PrefUnitSystemSelectorCBox.currentText())
        self.configParams.mImagePolicy = str(self.PrefMImagePolicyCBox.currentText())
        self.configParams.copyDataFilesToModelDir = self.PrefCopyDatafilesCheckBox.isChecked()
        txt = self.PrefParaViewComLedit.text()
        self.configParams.paraViewCommand = str(txt) 
        
        self.writeCustomization.emit(self.configParams)

    def readOutAllPrefsAndExit(self):


        self.configParams.windowParams['font_family'] = self.PrefWindowFontFamilyLedit.text()
        self.configParams.windowParams['font_size'] = str(str2int(self.PrefWindowFontSizeLedit.text(), 'Font Size (Window)'))
        self.configParams.figureParams['font_size'] = str(str2int(self.PrefCanvasFontSizeLedit.text(), 'Font Size (Image Canvas)'))
        self.configParams.unitSystem = str(self.PrefUnitSystemSelectorCBox.currentText())
        self.configParams.mImagePolicy = str(self.PrefMImagePolicyCBox.currentText())
        self.configParams.copyDataFilesToModelDir = self.PrefCopyDatafilesCheckBox.isChecked()
        txt = self.PrefParaViewComLedit.text()
        self.configParams.paraViewCommand = str(txt) 

        self.updateCustomization.emit(self.configParams, 1)
        self.accept()

class ImageViewerMainWindow(QtGui.QMainWindow, Ui_ImageViewerMainWindow):

    def __init__(self, parent = None):
        super(ImageViewerMainWindow, self).__init__(parent)
        self.setupUi(self)

        self.configParams = ConfigurationParams() 
        if os.path.exists(os.environ['HOME']+'/.artist.cfg'):
            self.configParamFileName = '.artist.cfg'
            self.loadDefaultConfigParams(True)
        else:
            self.configParamFileName = '.artist.cfg'
            self.loadDefaultConfigParams(False)
        

        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+Q"), self, self.close)
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+O"), self, self.browseImage)
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+right"), self, self.moveToNextTab)
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+left"), self, self.moveToPreviousTab)
        
        self.actionOpenImage.setShortcut('Ctrl+O')
        self.actionQuitIvan.setShortcut('Ctrl+Q')
        self.actionCustomize.setShortcut('Ctrl+P')

        self.ImMainTabWidget.setTabEnabled(1,True)
        self.ImMainTabWidget.setTabEnabled(2,True)

        if not PVDEnabled:
            self.ImMainTabWidget.removeTab(2)

        
        self.ImMainTabWidget.setCurrentIndex(0)
        self.setTabOrder(self.ImFitsNameLedit, self.ImBrowseImageButton)
        self.setTabOrder(self.ImBrowseImageButton, self.ImScaleLinRButton)
        self.setTabOrder(self.ImScaleLinRButton, self.ImScaleLogRButton)
        self.setTabOrder(self.ImScaleLogRButton, self.ImBeamConvCheckBox)
        self.setTabOrder(self.ImBeamConvCheckBox, self.ImBmajLedit)
        self.setTabOrder(self.ImBmajLedit, self.ImBminLedit)
        self.setTabOrder(self.ImBminLedit, self.ImPALedit)
        self.setTabOrder(self.ImPALedit, self.ImClipMaxLedit)
        self.setTabOrder(self.ImClipMaxLedit, self.ImClipMinLedit)
        self.setTabOrder(self.ImClipMinLedit, self.ImCMapColorMapSelectorCBox)
        self.setTabOrder(self.ImCMapColorMapSelectorCBox, self.ImCMapCLinesEnableCheckBox)
        self.setTabOrder(self.ImCMapCLinesEnableCheckBox, self.ImCMapCLinesMapSelectorCBox)
        self.setTabOrder(self.ImCMapCLinesMapSelectorCBox, self.ImCMapCLinesGridParButton)
        self.setTabOrder(self.ImCMapCLinesGridParButton, self.ImPlotPixelsSpectrumCheckBox)
        self.setTabOrder(self.ImPlotPixelsSpectrumCheckBox, self.ImIndexSlider)
        self.setTabOrder(self.ImIndexSlider, self.ImIndexLedit)
        self.setTabOrder(self.ImIndexLedit, self.ImVelLedit)
        
        
        self.setTabOrder(self.ImMMapMoment0RButton, self.ImMMapMoment1RButton)
        self.setTabOrder(self.ImMMapMoment1RButton, self.ImMMapMoment2RButton)
        self.setTabOrder(self.ImMMapMoment2RButton, self.ImMMapScaleLinRButton)
        self.setTabOrder(self.ImMMapScaleLinRButton, self.ImMMapScaleLogRButton)
        self.setTabOrder(self.ImMMapScaleLogRButton, self.ImMMapBeamConvCheckBox)
        self.setTabOrder(self.ImMMapBeamConvCheckBox, self.ImMMapBmajLedit)
        self.setTabOrder(self.ImMMapBmajLedit, self.ImMMapBminLedit)
        self.setTabOrder(self.ImMMapBminLedit, self.ImMMapPALedit)
        self.setTabOrder(self.ImMMapPALedit, self.ImMMapClipMaxLedit)
        self.setTabOrder(self.ImMMapClipMaxLedit, self.ImMMapClipMinLedit)
        self.setTabOrder(self.ImMMapClipMinLedit, self.ImMMapColorMapSelectorCBox)
        self.setTabOrder(self.ImMMapColorMapSelectorCBox, self.ImMMapCLinesEnableCheckBox)
        self.setTabOrder(self.ImMMapCLinesEnableCheckBox, self.ImMMapCLinesMapSelectorCBox)
        self.setTabOrder(self.ImMMapCLinesMapSelectorCBox, self.ImMMapCLinesGridParButton)
        
        
        self.setTabOrder(self.PVDiagPIndexSlider, self.PVDiagPIndexLedit)
        self.setTabOrder(self.PVDiagPIndexLedit, self.PVDiagPALedit)
        self.setTabOrder(self.PVDiagPALedit, self.PVDiagMMapSelector)
        self.setTabOrder(self.PVDiagMMapSelector, self.PVDiagLinScaleRButton)
        self.setTabOrder(self.PVDiagLinScaleRButton, self.PVDiagLogScaleRButton)
        self.setTabOrder(self.PVDiagLogScaleRButton, self.PVDiagClipMaxLedit)
        self.setTabOrder(self.PVDiagClipMaxLedit, self.PVDiagClipMaxResetButton)
        self.setTabOrder(self.PVDiagClipMaxResetButton, self.PVDiagClipMinLedit)
        self.setTabOrder(self.PVDiagClipMinLedit, self.PVDiagClipMinResetButton)
        self.setTabOrder(self.PVDiagClipMinResetButton, self.PVDiagColorMapCBox)
        self.setTabOrder(self.PVDiagColorMapCBox, self.PVDiagBeamConvCheckBox)
        self.setTabOrder(self.PVDiagBeamConvCheckBox, self.PVDiagBmajLedit)
        self.setTabOrder(self.PVDiagBmajLedit, self.PVDiagBminLedit)
        self.setTabOrder(self.PVDiagBminLedit, self.PVDiagBPALedit)

        self.HeaderDialog = HeaderDialog(self)

        self.imageData = ImageData()
        self.PixSpecWindow = mplFigurePixSpecMainWindow(self)

        self.ImBmajLedit.setText(str(self.imageData.image['bmaj']))
        self.ImBminLedit.setText(str(self.imageData.image['bmin']))
        self.ImPALedit.setText(str(self.imageData.image['pa']))
       
        self.ImMMapBmajLedit.setText(str(self.imageData.mmap['bmaj']))
        self.ImMMapBminLedit.setText(str(self.imageData.mmap['bmin']))
        self.ImMMapPALedit.setText(str(self.imageData.mmap['pa']))

        self.ImContourVarName = ['Channel Map', '0th Moment Map', '1st Moment Map', '2nd Moment Map'] 
        
        self.ImCMapCLinesMapSelectorCBox.addItems(self.ImContourVarName)
        self.ImMMapCLinesMapSelectorCBox.addItems(self.ImContourVarName[1:])

        self.inOpeningImage = False
        self.LContGrid={}
        self.FContGrid={}
        self.LContGridColor={}
        self.FContGridColor={}
        self.LContVarNameCMap = self.ImContourVarName[0]
        self.LContVarNameMMap = self.ImContourVarName[1]
        self.FContVarNameMMap = self.ImContourVarName[1]

        self.PVDiagPALedit.setText('23.0')
        self.PVDiagMMapSelector.addItems(['0th Moment Map', '1st Moment Map', '2nd Moment Map'])
        self.PVDiagMMapSelector.setCurrentIndex(0)
        self.connect(self.PVDiagMMapSelector,QtCore.SIGNAL("currentIndexChanged(int)"), self.updatePVDiagram)



        minval = 0.0
        maxval = 1.0
        nval = 10
        
        for ivar in self.ImContourVarName:
            self.LContGrid[ivar] = Grid(minval=minval, maxval=maxval, nval=nval, gridstyle='lin')
            self.FContGrid[ivar] = Grid(minval=minval, maxval=maxval, nval=nval, gridstyle='lin')
            self.LContGridColor[ivar] = GridColor(currentColorMap='casa',isCurrentSingleColor=True)
            self.FContGridColor[ivar] = GridColor(currentColorMap='casa')
        self.FContGridColor['PV Diagram'] = GridColor(currentColorMap='casa')
        self.ImCMapColorMapSelectorCBox.addItems(self.FContGridColor[self.ImContourVarName[0]].colorMaps.keys())
        self.ImMMapColorMapSelectorCBox.addItems(self.FContGridColor[self.ImContourVarName[0]].colorMaps.keys())
        self.PVDiagColorMapCBox.addItems(self.FContGridColor[self.ImContourVarName[0]].colorMaps.keys())
       
        if self.imageData.imageFName.strip()!='':
            self.imageData.readImage()

        if self.imageData.imexists:
            self.setCMapParams()
            self.updateChannelMap()
            self.mplFigureWidgetMMap.canvas.plotMomentMap(bunit=self.imageData.header['BUNIT'])
        
            self.setMMapParams()

            self.LContGrid={}
            self.FContGrid={}
            self.LContGridColor={}
            self.FContGridColor={}
            self.LContVarNameCMap = self.ImContourVarName[0]
            self.LContVarNameMMap = self.ImContourVarName[1]
            self.FContVarNameMMap = self.ImContourVarName[1]

            minval = self.imageData.image['data'].min()
            maxval = self.imageData.image['data'].max()
            nval = 30
           
            for ivar in self.ImContourVarName:
                self.LContGrid[ivar] = Grid(minval=minval, maxval=maxval, nval=nval, gridstyle='lin')
                self.FContGrid[ivar] = Grid(minval=minval, maxval=maxval, nval=nval, gridstyle='lin')
                self.LContGridColor[ivar] = GridColor()
                self.FContGridColor[ivar] = GridColor()
            
            self.ImCMapColorMapSelectorCBox.addItems(self.FContGridColor[self.ImContourVarName[0]].colorMaps.keys())
            self.ImMMapColorMapSelectorCBox.addItems(self.FContGridColor[self.ImContourVarName[0]].colorMaps.keys())

        self.ImCMapColorMapSelectorCBox.setCurrentIndex(9)
        self.ImMMapColorMapSelectorCBox.setCurrentIndex(9)
        self.PVDiagColorMapCBox.setCurrentIndex(9)


        self.PVDiagBmajLedit.setText('0.2')
        self.PVDiagBminLedit.setText('0.2')
        self.PVDiagBPALedit.setText('45.0')

        self.connect(self.actionOpenImage,QtCore.SIGNAL("triggered()"), self.browseImage)
        self.connect(self.actionShow_header,QtCore.SIGNAL("triggered()"), self.openHeaderDialog)
        self.connect(self.actionQuitIvan,QtCore.SIGNAL("triggered()"), self.close)
        self.connect(self.actionFont,QtCore.SIGNAL("triggered()"), self.openFontDialog)
        self.connect(self.actionCustomize,QtCore.SIGNAL("triggered()"), self.openCustomizeDialog)

        self.connect(self.ImBrowseImageButton,QtCore.SIGNAL("clicked()"), self.browseImage)
        
        self.connect(self.ImScaleLinRButton, QtCore.SIGNAL("clicked()"), self.updateChannelMap)
        self.connect(self.ImScaleLogRButton, QtCore.SIGNAL("clicked()"), self.updateChannelMap)
        self.connect(self.ImClipMinLedit,QtCore.SIGNAL("returnPressed()"), self.updateChannelMap)
        self.connect(self.ImClipMaxLedit,QtCore.SIGNAL("returnPressed()"), self.updateChannelMap)
        self.connect(self.ImIndexLedit,QtCore.SIGNAL("returnPressed()"), self.updateChannelMap)
        self.connect(self.ImCMapColorMapSelectorCBox,QtCore.SIGNAL("currentIndexChanged(int)"), self.updateChannelMap)
        self.connect(self.ImClipMaxResetButton,QtCore.SIGNAL("clicked()"), self.resetImClipMax)
        self.connect(self.ImClipMinResetButton,QtCore.SIGNAL("clicked()"), self.resetImClipMin)
        
        self.connect(self.ImIndexSlider,QtCore.SIGNAL("valueChanged(int)"), self.sliderMoved)
        
        self.connect(self.ImBmajLedit,QtCore.SIGNAL("returnPressed()"), self.switchConvImage)
        self.connect(self.ImBminLedit,QtCore.SIGNAL("returnPressed()"), self.switchConvImage)
        self.connect(self.ImPALedit,QtCore.SIGNAL("returnPressed()"), self.switchConvImage)
        self.connect(self.ImBeamConvCheckBox,QtCore.SIGNAL("clicked()"), self.switchConvImage)

        self.connect(self.ImCMapCLinesEnableCheckBox,QtCore.SIGNAL("clicked()"), self.updateChannelMap)
        self.connect(self.ImCMapCLinesMapSelectorCBox,QtCore.SIGNAL("currentIndexChanged(int)"), self.updateChannelMap)
        self.connect(self.ImCMapCLinesGridParButton,QtCore.SIGNAL("clicked()"), self.openCMapLGridDialog)

        self.connect(self.CMapStokesAxisSelectorCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.changeStokesAxis)
        
        self.connect(self.ImMMapScaleLinRButton, QtCore.SIGNAL("clicked()"), self.updateMomentMap)
        self.connect(self.ImMMapScaleLogRButton, QtCore.SIGNAL("clicked()"), self.updateMomentMap)
        self.connect(self.ImMMapThresholdSelectorLedit,QtCore.SIGNAL("returnPressed()"), self.updateMomentMapClips)
        self.connect(self.ImMMapChannelSelectorLedit,QtCore.SIGNAL("returnPressed()"), self.updateMomentMapClips)
        self.connect(self.ImMMapUpdateButton,QtCore.SIGNAL("clicked()"), self.updateMomentMapClips)
        
        self.connect(self.ImMMapClipMinLedit,QtCore.SIGNAL("returnPressed()"), self.updateMomentMap)
        self.connect(self.ImMMapClipMaxLedit,QtCore.SIGNAL("returnPressed()"), self.updateMomentMap)
        self.connect(self.ImMMapColorMapSelectorCBox,QtCore.SIGNAL("currentIndexChanged(int)"), self.updateMomentMap)
        self.connect(self.ImMMapClipMaxResetButton,QtCore.SIGNAL("clicked()"), self.resetMMapClipMax)
        self.connect(self.ImMMapClipMinResetButton,QtCore.SIGNAL("clicked()"), self.resetMMapClipMin)
        
        self.connect(self.ImMMapBmajLedit,QtCore.SIGNAL("returnPressed()"), self.updateMomentMap)
        self.connect(self.ImMMapBminLedit,QtCore.SIGNAL("returnPressed()"), self.updateMomentMap)
        self.connect(self.ImMMapPALedit,QtCore.SIGNAL("returnPressed()"), self.updateMomentMap)
        self.connect(self.ImMMapBeamConvCheckBox,QtCore.SIGNAL("clicked()"), self.updateMomentMap)
        
        self.connect(self.ImMMapMoment0RButton,QtCore.SIGNAL("clicked()"), self.updateMomentMap)
        self.connect(self.ImMMapMoment1RButton,QtCore.SIGNAL("clicked()"), self.updateMomentMap)
        self.connect(self.ImMMapMoment2RButton,QtCore.SIGNAL("clicked()"), self.updateMomentMap)
        self.connect(self.ImMMapCLinesEnableCheckBox,QtCore.SIGNAL("clicked()"), self.updateMomentMap)
        self.connect(self.ImMMapCLinesMapSelectorCBox,QtCore.SIGNAL("currentIndexChanged(int)"), self.updateMomentMap)
        self.connect(self.ImMMapCLinesGridParButton,QtCore.SIGNAL("clicked()"), self.openMMapLGridDialog)



        self.connect(self.ImPlotPixelsSpectrumCheckBox,QtCore.SIGNAL("clicked()"), self.enablePixSpecWindow)
        self.connect(self.mplFigureWidgetCMap.canvas, QtCore.SIGNAL("ClickedOnCanvas(float, float, int)"),
                self.plotPixelSpectrum)

        self.mplFigureWidgetPVDiag.toolbar.writePVDiagramToFits.connect(self.imageData.writeImage)


        self.connect(self.PVDiagPIndexSlider, QtCore.SIGNAL("valueChanged(int)"), self.updatePVDiagramToNewIndex)
        self.connect(self.PVDiagPALedit, QtCore.SIGNAL("returnPressed()"), self.updatePVDiagram)
        self.connect(self.PVDiagClipMaxLedit, QtCore.SIGNAL("returnPressed()"), self.updatePVDiagramNewColors)
        self.connect(self.PVDiagClipMinLedit, QtCore.SIGNAL("returnPressed()"), self.updatePVDiagramNewColors)
        self.connect(self.PVDiagClipMaxResetButton, QtCore.SIGNAL("clicked()"), self.resetPVDiagClipMax)
        self.connect(self.PVDiagClipMinResetButton, QtCore.SIGNAL("clicked()"), self.resetPVDiagClipMin)
        self.connect(self.PVDiagPIndexLedit, QtCore.SIGNAL("returnPressed()"), self.updatePVDiagramPosind)

        self.connect(self.PVDiagLinScaleRButton, QtCore.SIGNAL("clicked()"), self.updatePVDiagramNewColors)
        self.connect(self.PVDiagLogScaleRButton, QtCore.SIGNAL("clicked()"), self.updatePVDiagramNewColors)
        self.connect(self.PVDiagColorMapCBox, QtCore.SIGNAL("currentIndexChanged(int)"), self.updatePVDiagramNewColors)
        self.connect(self.PVDiagUpdateButton, QtCore.SIGNAL("clicked()"), self.updatePVDiagram)

        self.connect(self.PVDiagBmajLedit,QtCore.SIGNAL("returnPressed()"), self.switchConvPVDiag)
        self.connect(self.PVDiagBminLedit,QtCore.SIGNAL("returnPressed()"), self.switchConvPVDiag)
        self.connect(self.PVDiagPALedit,QtCore.SIGNAL("returnPressed()"), self.switchConvPVDiag)
        self.connect(self.PVDiagBeamConvCheckBox,QtCore.SIGNAL("clicked()"), self.switchConvPVDiag)

    def writeDefaultConfigParams(self, configParams=None):

        if configParams!=None:
            configParams.writeConfigurationParamsToFile(fName=self.configParamFileName)

    def loadDefaultConfigParams(self, readParFile=True):
       
        if readParFile:
            self.configParams.readConfigurationParamsFromFile(fName=self.configParamFileName)
        self.updateImageViewerCustomization(self.configParams, 1)
    

    def moveToPreviousTab(self):
        ind = self.ImMainTabWidget.currentIndex()
        if ind==0: ind = 3
        self.ImMainTabWidget.setCurrentIndex((ind-1)%3)

    def moveToNextTab(self):

        ind = self.ImMainTabWidget.currentIndex()
        self.ImMainTabWidget.setCurrentIndex((ind+1)%3)


    def updateFigureMMapWithNewGrid(self, grid=None, gridcolor=None, varName=None, filledContours=0):

        if self.imageData.imexists:
            varName = str(varName)
            if filledContours==0:
                self.LContGrid[varName] = cp.deepcopy(grid)
                self.LContGridColor[varName] = cp.deepcopy(gridcolor)
                self.updateMomentMap()

    def updateFigureCMapWithNewGrid(self, grid=None, gridcolor=None, varName=None, filledContours=0):

        if self.imageData.imexists:
            varName = str(varName)
            if filledContours==0:
                self.LContGrid[varName] = cp.deepcopy(grid)
                self.LContGridColor[varName] = cp.deepcopy(gridcolor)
                self.updateChannelMap()
        
    def getGridMMap(self, varName=None):

        if self.imageData.imexists:
            if varName=='0th Moment Map':
                databounds = [self.imageData.mmap['data'][:,:,0].min(), self.imageData.mmap['data'][:,:,0].max()]
            if varName=='1st Moment Map':
                databounds = [self.imageData.mmap['data'][:,:,1].min(), self.imageData.mmap['data'][:,:,1].max()]
            if varName=='2nd Moment Map':
                databounds = [self.imageData.mmap['data'][:,:,2].min(), self.imageData.mmap['data'][:,:,2].max()]

            log.debug('Data bounds for moment map grid :'+(('%.9e %.9e')%(databounds[0], databounds[1])))
            dum = GridDialog(self, grid=self.LContGrid[varName], varName=varName, \
                    databounds=databounds, gridcolor=self.LContGridColor[varName], windowstyle=0)
            dum.ApplyGrid.connect(self.updateFigureMMapWithNewGrid)
            dum.exec_()
            dum.ApplyGrid.disconnect(self.updateFigureMMapWithNewGrid)

    def getGridCMap(self, varName=None):

        if self.imageData.imexists:
            if varName=='Channel Map':
                if not self.imageData.image['plotconv']:
                    databounds = [self.imageData.image['data'].min(), self.imageData.image['data'].max()]
                else:
                    databounds = [self.imageData.image['cdata'].min(), self.imageData.image['cdata'].max()]

            if varName=='0th Moment Map':
                if not self.imageData.image['plotconv']:
                    databounds = [self.imageData.mmap['data'][:,:,0].min(), self.imageData.mmap['data'][:,:,0].max()]
                else:
                    databounds = [self.imageData.mmap['cdata'][:,:,0].min(), self.imageData.mmap['cdata'][:,:,0].max()]

            if varName=='1st Moment Map':
                if not self.imageData.image['plotconv']:
                    databounds = [self.imageData.mmap['data'][:,:,1].min(), self.imageData.mmap['data'][:,:,1].max()]
                else:
                    databounds = [self.imageData.mmap['cdata'][:,:,1].min(), self.imageData.mmap['cdata'][:,:,1].max()]

            if varName=='2nd Moment Map':
                if not self.imageData.image['plotconv']:
                    databounds = [self.imageData.mmap['data'][:,:,2].min(), self.imageData.mmap['data'][:,:,2].max()]
                else:
                    databounds = [self.imageData.mmap['cdata'][:,:,2].min(), self.imageData.mmap['cdata'][:,:,2].max()]

            log.debug('Data bounds for channel map grid :'+('%.9e %.9e')%(databounds[0], databounds[1]))

            dum = GridDialog(self, grid=self.LContGrid[varName], varName=varName, \
                    databounds=databounds, gridcolor=self.LContGridColor[varName], windowstyle=0)
            dum.ApplyGrid.connect(self.updateFigureCMapWithNewGrid)
            dum.exec_()
            dum.ApplyGrid.disconnect(self.updateFigureCMapWithNewGrid)

    def updateImageViewerCustomization(self, configParams, doUpdate):

        self.configParams = cp.deepcopy(configParams)
        if doUpdate:
            self.setStyleSheet('font-family: '+self.configParams.windowParams['font_family']+\
                    '; font-size: '+self.configParams.windowParams['font_size']+'pt;')

            self.mplFigureWidgetCMap.canvas.updateAxisParams(self.configParams.figureParams)
            self.mplFigureWidgetMMap.canvas.updateAxisParams(self.configParams.figureParams)
            self.mplFigureWidgetPVDiag.canvas.updateAxisParams(self.configParams.figureParams)
            self.mplFigureWidgetPVDiagMMap.canvas.updateAxisParams(self.configParams.figureParams)

    def openCustomizeDialog(self):

        dum = ImageViewerCustomizeDialog(self,self.configParams)
        dum.updateCustomization.connect(self.updateImageViewerCustomization)
        dum.writeCustomization.connect(self.writeDefaultConfigParams)
        dum.exec_()
        dum.updateCustomization.disconnect(self.updateImageViewerCustomization)
        dum.writeCustomization.disconnect(self.writeDefaultConfigParams)

    def openFontDialog(self):
        
        oldFont = self.font()
        dum = QtGui.QFontDialog()
        dum.setCurrentFont(oldFont)
        dum.exec_()
        if dum.result()==1:
            newFont = dum.selectedFont()
            family = str(newFont.family())
            psize = str(newFont.pointSize()).strip()
           
            self.setStyleSheet('font-family: '+family+'; font-size: '+psize+';')

    def openHeaderDialog(self):

        self.HeaderDialog.exec_()

    def openCMapLGridDialog(self):

        if self.imageData.imexists:
        
            self.LContVarNameCMap = str(self.ImCMapCLinesMapSelectorCBox.currentText())     
            self.getGridCMap(varName=self.LContVarNameCMap)
    
    def openMMapLGridDialog(self):
        if self.imageData.imexists:
            self.LContVarNameMMap = str(self.ImMMapCLinesMapSelectorCBox.currentText())     
            self.getGridMMap(varName=self.LContVarNameMMap)

    def plotPixelSpectrum(self, dataX=None, dataY=None, button=None):
        if self.imageData.imexists:
            if self.mplFigureWidgetCMap.canvas.enablePixSpec:
                log.debug('Plotting spectrum in a pixel')
                if button==1:
                    if self.imageData.image['data'].shape>1:
                        ix = np.where(self.imageData.image['crd_xi']<dataX)
                        iy = np.where(self.imageData.image['crd_yi']>=dataY)
                       
                        ix = ix[0][0]-1
                        iy = iy[0][0]-1
                        iy2 = self.imageData.image['crd_y'].shape[0]-iy-1
                    
                   

                        
                        spec = self.imageData.image['data'][self.imageData.image['stokes_index'],:,iy2,ix]
                        if self.imageData.image['plotconv']:
                            spec = self.imageData.image['cdata'][self.imageData.image['stokes_index'],:,iy2,ix]

                        self.PixSpecWindow.mplFigureWidgetPixSpec.canvas.plotData(\
                                self.imageData.image['crd_vel'],\
                                spec, xtitle='Velocity [km/s]', ytitle=self.imageData.header['BUNIT'])
                        
                        self.PixSpecWindow.setVisible(True)


    def enablePixSpecWindow(self):
        self.mplFigureWidgetCMap.canvas.enablePixSpec = self.ImPlotPixelsSpectrumCheckBox.isChecked()

    def sliderMoved(self, value):

        if self.imageData.imexists:
            self.imageData.image['ifreq'] = self.ImIndexSlider.value()
            txt = str(self.imageData.image['crd_vel'][self.imageData.image['ifreq']])
            self.ImVelLedit.setText(txt)
            txt = str(self.imageData.image['ifreq'])
            self.ImIndexLedit.setText(txt)
            self.updateChannelMapToNewChannel()

    def readImage(self):

        log.debug('Reading image')
        self.imageData.readImage()
        if not self.imageData.imexists:
            return
       
        self.inOpeningImage = True
       

        nstokes = self.imageData.image['fdata'].shape[0]

        self.CMapStokesAxisSelectorCBox.clear()
        dum = np.arange(nstokes, dtype=int)
        sdum = [str(i) for i in dum]
        self.CMapStokesAxisSelectorCBox.addItems(sdum)
        
        if self.imageData.image['crd_vel'].shape[0]==1: 
            self.ImMainTabWidget.setTabEnabled(1, False)
            self.ImMainTabWidget.setTabEnabled(2, False)
            self.PlotPixelSpectrumFrame.setVisible(False)
            self.ImIndexLabel.setEnabled(False)
            self.ImIndexLedit.setEnabled(False)
            self.ImIndexSlider.setEnabled(False)
            self.ImVelLabel.setEnabled(False)
            self.ImVelLedit.setEnabled(False)
        else:
            self.ImMainTabWidget.setTabEnabled(1, True)
            self.ImMainTabWidget.setTabEnabled(2, True)
            self.PlotPixelSpectrumFrame.setVisible(True)
            self.ImIndexLabel.setEnabled(True)
            self.ImIndexLedit.setEnabled(True)
            self.ImIndexSlider.setEnabled(True)
            self.ImVelLabel.setEnabled(True)
            self.ImVelLedit.setEnabled(True)

        if self.imageData.image['data'].shape[0]>1:
            self.CMapStokesFrame.setVisible(True)
            self.CMapStokesAxisSelectorCBox.clear()
            dum = np.arange(self.imageData.image['data'].shape[0], dtype=int)
            sdum = [str(i) for i in dum]
            self.CMapStokesAxisSelectorCBox.addItems(sdum)
        else:
            self.CMapStokesFrame.setVisible(False)



        if len(self.imageData.image['data'].shape)==4:
            nfreq = self.imageData.image['data'].shape[1]
        else:
            nfreq = self.imageData.image['data'].shape[0]
        self.ImIndexSlider.setMinimum(0)
        self.ImIndexSlider.setMaximum(nfreq-1)
        self.ImIndexSlider.setValue(0)


        if len(self.imageData.image['data'].shape)>3:
            self.PVDiagPIndexSlider.setMinimum(0)
            lod = len(self.imageData.image['data'].shape)
            self.PVDiagPIndexSlider.setMaximum(self.imageData.image['data'].shape[lod-2])
            self.ImIndexSlider.setValue(0)

        self.ImFitsNameLedit.setText(self.imageData.imageFName)

        self.setCMapParams()
        self.imageData.imconv()
        if self.imageData.image['crd_vel'].shape[0]>1:
            self.inOpeningImage = True
            self.ImCMapCLinesMapSelectorCBox.clear()
            self.ImContourVarName = ['Channel Map', '0th Moment Map', '1st Moment Map', '2nd Moment Map'] 
            self.ImCMapCLinesMapSelectorCBox.addItems(self.ImContourVarName)
           
            self.inOpeningImage = False
            
            self.LContGrid[self.ImContourVarName[0]] = Grid(minval=self.imageData.image['data'].min(), maxval=self.imageData.image['data'].max(), \
                    nval=10,gridstyle='lin')

        else:
            self.ImCMapCLinesMapSelectorCBox.clear()
            self.ImContourVarName = ['Channel Map']
            self.ImCMapCLinesMapSelectorCBox.addItems(self.ImContourVarName)



        self.HeaderDialog.clear()
        self.HeaderDialog.fillupHeader(self.imageData.header)
        self.inOpeningImage = False


    def switchConvImage(self):
    
        if self.imageData.imexists:
       
            log.debug('Doing beam convolution with bmaj:'+self.ImBmajLedit.text()+' bmin: '+self.ImBminLedit.text()+' PA: '+self.ImPALedit.text())
            
            if self.ImBeamConvCheckBox.isChecked():
                self.imageData.image['plotconv'] = True

                do_conv = False

                if (len(self.imageData.image['cdata'].shape)<2): do_conv = True

                val = str2float(self.ImBmajLedit.text(), 'Major axis (Channel maps)')
                if (val!=self.imageData.image['bmaj']): 
                    do_conv = True
                    self.imageData.image['bmaj'] = val

                val = str2float(self.ImBminLedit.text(), 'Minor axis (Channel maps)')
                if (val!=self.imageData.image['bmin']): 
                    do_conv = True
                    self.imageData.image['bmin'] = val

                val = str2float(self.ImPALedit.text(), 'Position angle (Channel maps)')
                if (val!=self.imageData.image['pa']): 
                    do_conv = True
                    self.imageData.image['pa'] = val

                if do_conv:
                    self.imageData.imconv()
                    if self.imageData.image['data'].shape[1]>1: self.imageData.makeMomentMap(conv=True)
                
                self.ImMMapBmajLedit.setText(str(self.ImBmajLedit.text()))
                self.ImMMapBminLedit.setText(str(self.ImBminLedit.text()))
                self.ImMMapPALedit.setText(str(self.ImPALedit.text()))
                do_conv = False
                if self.imageData.mmap['bmaj']!=self.imageData.image['bmaj']: do_conv=True
                if self.imageData.mmap['bmin']!=self.imageData.image['bmin']: do_conv=True
                if self.imageData.mmap['pa']!=self.imageData.image['pa']: do_conv=True
                    
            else:
                self.imageData.image['plotconv'] = False

            self.inOpeningImage = True
            self.resetImClipMin()
            self.resetImClipMax()
            self.inOpeningImage = False
            self.updateChannelMap()

    def browseImage(self):

        imageFName = QtGui.QFileDialog.getOpenFileName(self, 'Open file', options=QtGui.QFileDialog.DontUseNativeDialog)
        self.imageData.imageFName = str(imageFName)
        log.debug('Image file name selected :'+self.imageData.imageFName)

        if self.imageData.imageFName.strip()!='':
            self.readImage()
            self.updateChannelMap()

            if len(self.imageData.image['data'].shape)>2:
                if self.imageData.image['data'].shape[1]>1:
                    for i in range(2):
                        self.imageData.mmap['clipMin'][i] = self.imageData.mmap['data'][:,:,i].min()
                        self.imageData.mmap['clipMax'][i] = self.imageData.mmap['data'][:,:,i].max()
                    self.imageData.mmap['clipMin'][2] = np.sqrt(self.imageData.mmap['data'][:,:,2].min())
                    self.imageData.mmap['clipMax'][2] = np.sqrt(self.imageData.mmap['data'][:,:,2].max())
                    self.ImMMapClipMaxLedit.setText(str(self.imageData.mmap['clipMax'][0]))
                    self.ImMMapClipMinLedit.setText(str(self.imageData.mmap['clipMin'][0]))
                    self.updateMomentMap()

                    self.inOpeningImage=True
                    self.updatePVDiagram()
                    self.inOpeningImage=False
                    self.imageData.pvdiag['clipMin']  = self.imageData.pvdiag['data'].min()
                    self.imageData.pvdiag['clipMax']  = self.imageData.pvdiag['data'].max()
                    self.PVDiagClipMinLedit.setText(str(self.imageData.pvdiag['clipMin']))
                    self.PVDiagClipMaxLedit.setText(str(self.imageData.pvdiag['clipMax']))
        

    def setCMapParams(self):

        if self.imageData.imexists:
            if  self.imageData.image['enableCLines']:
                self.ImCMapCLinesEnableCheckBox.setChecked()

            log.debug('Clipping the channel maps:')
            txt = str(self.imageData.image['clipMin'])
            log.debug('min: '+txt)
            self.ImClipMinLedit.setText(txt)
            txt = str(self.imageData.image['clipMax'])
            log.debug('max: '+txt)
            self.ImClipMaxLedit.setText(txt)
            txt = str(self.imageData.image['ifreq'])
            self.ImIndexLedit.setText(txt)
            txt = str(self.imageData.image['crd_vel'][self.imageData.image['ifreq']])
            self.ImVelLedit.setText(txt)
    
    def resetImClipMax(self):

        if self.imageData.imexists:
            if self.ImBeamConvCheckBox.isChecked():
               val = self.imageData.image['cdata'].max()
            else:
               val = self.imageData.image['data'].max()
            
            self.ImClipMaxLedit.setText(str(val))
            
            self.imageData.image['clipMax'] = val
            self.updateChannelMap()

    def resetImClipMin(self):

        if self.imageData.imexists:
            if self.ImBeamConvCheckBox.isChecked():
               val = self.imageData.image['cdata'].min()
            else:
               val = self.imageData.image['data'].min()
           
            self.ImClipMinLedit.setText(str(val))
            self.imageData.image['clipMin'] = val
            self.updateChannelMap()

    def readOutCMapParams(self):
    
        if self.imageData.imexists:
            self.imageData.image['enableCLines'] = self.ImCMapCLinesEnableCheckBox.isChecked()
            if self.ImScaleLinRButton.isChecked():self.imageData.image['logscale']=False
            if self.ImScaleLogRButton.isChecked():self.imageData.image['logscale']=True
            
            self.LContVarNameCMap=str(self.ImCMapCLinesMapSelectorCBox.currentText())
            self.FContGridColor['Channel Map'].currentColorMap = str(self.ImCMapColorMapSelectorCBox.currentText())

            if self.LContVarNameCMap=='Channel Map':
                if self.imageData.image['plotconv']:
                    self.LCdata = self.imageData.image['cdata'][self.imageData.image['stokes_index'], self.imageData.image['ifreq'],:,:]
                else:
                    self.LCdata = self.imageData.image['data'][self.imageData.image['stokes_index'], self.imageData.image['ifreq'],:,:]

            if self.LContVarNameCMap=='0th Moment Map':
                if self.imageData.image['plotconv']:
                    self.imageData.mmap['bmaj'] = self.imageData.image['bmaj']
                    self.imageData.mmap['bmin'] = self.imageData.image['bmin']
                    self.imageData.mmap['pa'] = self.imageData.image['pa']

                    self.imageData.imconv()
                    self.LCdata = self.imageData.mmap['cdata'][:,:,0]
                else:
                    self.LCdata = self.imageData.mmap['data'][:,:,0]
            if self.LContVarNameCMap=='1st Moment Map':
                if self.imageData.image['plotconv']:
                    self.imageData.mmap['bmaj'] = self.imageData.image['bmaj']
                    self.imageData.mmap['bmin'] = self.imageData.image['bmin']
                    self.imageData.mmap['pa'] = self.imageData.image['pa']

                    self.imageData.imconv()
                    self.LCdata = self.imageData.mmap['cdata'][:,:,1]
                else:
                    self.LCdata = self.imageData.mmap['data'][:,:,1]
            if self.LContVarNameCMap=='2nd Moment Map':
                if self.imageData.image['plotconv']:
                    self.imageData.mmap['bmaj'] = self.imageData.image['bmaj']
                    self.imageData.mmap['bmin'] = self.imageData.image['bmin']
                    self.imageData.mmap['pa'] = self.imageData.image['pa']

                    self.imageData.imconv()
                    self.LCdata = self.imageData.mmap['cdata'][:,:,2]
                else:
                    self.LCdata = self.imageData.mmap['data'][:,:,2]


            val = str2float(self.ImClipMinLedit.text(), 'Clip min. (Channel maps)') 
            self.imageData.image['clipMin'] = val

            val = str2float(self.ImClipMaxLedit.text(), 'Clip max. (Channel maps)') 
            self.imageData.image['clipMax'] = val

            val = str2int(self.ImIndexLedit.text(), 'Channel Index (Channel maps)') 
            self.imageData.image['ifreq'] = val


    def changeStokesAxis(self, ind):

        self.imageData.image['stokes_index']  = ind
        if self.imageData.image['fdata'].shape[1]>1:
            self.updateChannelMap()
            self.updateMomentMap()
        else: 
            self.imageData.image['stokes_index']=ind
            self.updateChannelMap()
    
    def resetPVDiagClipMin(self):

        if self.imageData.imexists:
            if self.imageData.pvdiag['plotconv']:
                val = self.imageData.pvdiag['cdata'].min()
            else:
                val = self.imageData.pvdiag['data'].min()

            self.PVDiagClipMinLedit.setText(str(val))
            if self.ImMainTabWidget.isTabEnabled(2):
                self.imageData.image['clipMin'] = val
                self.PVDiagClipMinLedit.setText(str(val))
                self.updatePVDiagram()

    def resetPVDiagClipMax(self):

        if self.imageData.imexists:
            if self.imageData.pvdiag['plotconv']:
                val = self.imageData.pvdiag['cdata'].max()
            else:
                val = self.imageData.pvdiag['data'].max()
            self.PVDiagClipMaxLedit.setText(str(val))
            if self.ImMainTabWidget.isTabEnabled(2):
                self.imageData.image['clipMax'] = val
                self.PVDiagClipMaxLedit.setText(str(val))
                self.updatePVDiagram()

    def updatePVDiagramNewColors(self):

        if not PVDEnabled: return


        self.FContGridColor['PV Diagram'].currentColorMap = str(self.PVDiagColorMapCBox.currentText())
        if self.PVDiagLinScaleRButton.isChecked(): self.imageData.pvdiag['logscale']=False
        if self.PVDiagLogScaleRButton.isChecked(): self.imageData.pvdiag['logscale']=True

        if self.inOpeningImage:
            if self.imageData.pvdiag['plotconv']:
                val = self.imageData.pvdiag['cdata'].max()
                self.PVDiagClipMaxLedit.setText(str(val))
                val = self.imageData.pvdiag['cdata'].min()
                self.PVDiagClipMinLedit.setText(str(val))
            else:
                val = self.imageData.pvdiag['data'].max()
                self.PVDiagClipMaxLedit.setText(str(val))
                val = self.imageData.pvdiag['data'].min()
                self.PVDiagClipMinLedit.setText(str(val))
       

        nmom = self.PVDiagMMapSelector.currentIndex()
        txt = self.PVDiagClipMaxLedit.text()
        if str(txt).strip()=='':
            self.imageData.pvdiag['clipMax']  = self.imageData.pvdiag['data'].max()
            self.PVDiagClipMaxLedit.setText(str(self.imageData.pvdiag['clipMax']))
            txt = self.PVDiagClipMaxLedit.text()
           
        self.imageData.pvdiag['clipMax'] = str2float(txt, 'Clip Max. (PV Diagram)')
        txt = self.PVDiagClipMinLedit.text()
        if str(txt).strip()=='':
            self.imageData.pvdiag['clipMin']  = self.imageData.pvdiag['data'].min()
            self.PVDiagClipMinLedit.setText(str(self.imageData.pvdiag['clipMin']))
            txt = self.PVDiagClipMinLedit.text()
        self.imageData.pvdiag['clipMin'] = str2float(txt, 'Clip Min. (PV Diagram)')


        self.mplFigureWidgetPVDiag.canvas.plotPVDiagram(imageData=self.imageData, clineVarName=self.LContVarNameCMap, LContGrid=self.LContGrid,\
                    LContGridColor=self.LContGridColor, FContGridColor=self.FContGridColor['PV Diagram'])
      
        if len(self.imageData.mmap['data'].shape)>=2:
            self.mplFigureWidgetPVDiagMMap.canvas.plotMomentMap(imageData=self.imageData, clineVarName=self.LContVarNameMMap, clineVarID=self.LContVarIDMMap, \
                LContGrid=self.LContGrid, LContGridColor=self.LContGridColor, FContGridColor=self.FContGridColor[self.FContVarNameMMap],\
                bunit=self.imageData.header['BUNIT'], PVDiagLineOverlay=True, nmom_inp=nmom)

        self.PVDiagPIndexLedit.setText(str(self.imageData.pvdiag['posind']))


    def updatePVDiagram(self):

        if not PVDEnabled: return

        if self.imageData.imexists:
            txt = str(self.PVDiagPALedit.text())
            
            dum = str2float(txt, 'Position angle (PV Diagram)')
#            if (self.imageData.pvdiag['posang']!=90.-dum):
#                self.imageData.pvdiag['posang'] = 90.-dum
#                do_pvdiagram = True
#                self.imageData.makePVDiagram()
#                self.imageData.makePVDiagram(conv=True)
            self.imageData.pvdiag['posang'] = 90.-dum
            self.imageData.makePVDiagram()
            self.imageData.makePVDiagram(conv=True)
            self.imageData.pvdiag['posind'] = self.PVDiagPIndexSlider.value()
            self.FContGridColor['PV Diagram'].currentColorMap = str(self.PVDiagColorMapCBox.currentText())

            
            if self.PVDiagLinScaleRButton.isChecked(): self.imageData.pvdiag['logscale']=False
            if self.PVDiagLogScaleRButton.isChecked(): self.imageData.pvdiag['logscale']=True

            if self.inOpeningImage:
                if self.imageData.pvdiag['plotconv']:
                    val = self.imageData.pvdiag['cdata'].max()
                    self.PVDiagClipMaxLedit.setText(str(val))
                    val = self.imageData.pvdiag['cdata'].min()
                    self.PVDiagClipMinLedit.setText(str(val))
                else:
                    val = self.imageData.pvdiag['data'].max()
                    self.PVDiagClipMaxLedit.setText(str(val))
                    val = self.imageData.pvdiag['data'].min()
                    self.PVDiagClipMinLedit.setText(str(val))
     
            self.imageData.makePVDiagCrdaxes()
            self.PVDiagPIndexSlider.setMinimum(0)
            self.PVDiagPIndexSlider.setMaximum(self.imageData.pvdiag['data'].shape[2]-1)
           
            self.ImMMapClipMinLedit.setText(txt)

            nmom = self.PVDiagMMapSelector.currentIndex()
                    
            txt = self.PVDiagClipMaxLedit.text()
            if str(txt).strip()=='':
                self.imageData.pvdiag['clipMax']  = self.imageData.pvdiag['data'].max()
                self.PVDiagClipMaxLedit.setText(str(self.imageData.pvdiag['clipMax']))
                txt = self.PVDiagClipMaxLedit.text()
               
            self.imageData.pvdiag['clipMax'] = str2float(txt, 'Clip Max. (PV Diagram)')
            txt = self.PVDiagClipMinLedit.text()
            if str(txt).strip()=='':
                self.imageData.pvdiag['clipMin']  = self.imageData.pvdiag['data'].min()
                self.PVDiagClipMinLedit.setText(str(self.imageData.pvdiag['clipMin']))
                txt = self.PVDiagClipMinLedit.text()
            self.imageData.pvdiag['clipMin'] = str2float(txt, 'Clip Min. (PV Diagram)')


            self.mplFigureWidgetPVDiag.canvas.plotPVDiagram(imageData=self.imageData, clineVarName=self.LContVarNameCMap, LContGrid=self.LContGrid,\
                        LContGridColor=self.LContGridColor, FContGridColor=self.FContGridColor['PV Diagram'])
          
            if len(self.imageData.mmap['data'].shape)>=2:
                self.mplFigureWidgetPVDiagMMap.canvas.plotMomentMap(imageData=self.imageData, clineVarName=self.LContVarNameMMap, clineVarID=self.LContVarIDMMap, \
                    LContGrid=self.LContGrid, LContGridColor=self.LContGridColor, FContGridColor=self.FContGridColor[self.FContVarNameMMap],\
                    bunit=self.imageData.header['BUNIT'], PVDiagLineOverlay=True, nmom_inp=nmom)

            self.PVDiagPIndexLedit.setText(str(self.imageData.pvdiag['posind']))

    def updatePVDiagramPosind(self):

        txt = self.PVDiagPIndexLedit.text()
        self.imageData.pvdiag['posind'] = str2int(txt, 'Position index (PV Diagram)')
        self.PVDiagPIndexSlider.setValue(self.imageData.pvdiag['posind']) 
        self.updatePVDiagramToNewIndex()

    def updatePVDiagramToNewIndex(self):

            self.imageData.pvdiag['posind'] = self.PVDiagPIndexSlider.value()
            self.PVDiagPIndexLedit.setText(str(self.imageData.pvdiag['posind']))

            if self.imageData.pvdiag['plotconv']:
                data = self.imageData.pvdiag['cdata'][:,self.imageData.pvdiag['posind'],::-1].clip(self.imageData.pvdiag['clipMin'],self.imageData.pvdiag['clipMax'])
            else:
                data = self.imageData.pvdiag['data'][:,self.imageData.pvdiag['posind'],::-1].clip(self.imageData.pvdiag['clipMin'],self.imageData.pvdiag['clipMax'])

            if self.imageData.pvdiag['logscale']: data = np.log10(data)

            self.mplFigureWidgetPVDiag.canvas.pvd_plot.set_array(data)

            self.mplFigureWidgetPVDiag.canvas.fig.canvas.draw()
            
            ixc = (self.imageData.mmap['crd_x'].max() + self.imageData.mmap['crd_x'].min()) * 0.5
            iyc = (self.imageData.mmap['crd_y'].max() + self.imageData.mmap['crd_y'].min()) * 0.5 

#
#            r      = self.imageData.pvdiag['crd_x']
#            if self.imageData.pvdiag['crd_y'].max()>self.imageData.pvdiag['crd_x'].max():
#                r = self.imageData.pvdiag['crd_y']
#            y = (x-ixc) * np.tan(self.imageData.pvdiag['posang']/180.*np.pi) + ixc + \
#                     (r[self.imageData.pvdiag['posind']-1] - ixc) / np.cos(self.imageData.pvdiag['posang']/180.*np.pi)
            
            
            
            x = self.imageData.mmap['crd_x'][::-1]
            r      = self.imageData.pvdiag['crd_x']
            if self.imageData.pvdiag['crd_y'].max()>self.imageData.pvdiag['crd_x'].max():
                r = self.imageData.pvdiag['crd_y']
        
            dr = abs(r[1]-r[0]) * (self.imageData.pvdiag['posind']-(float(r.shape[0])*0.5))
            pa_rad = self.imageData.pvdiag['posang']/180.*np.pi
            y = (iyc + dr*np.cos(pa_rad)) + np.tan(pa_rad) * (x - ixc + dr*np.sin(pa_rad))
           

            if len(self.imageData.mmap['data'].shape)>=2:
                if self.mplFigureWidgetPVDiagMMap.canvas.pvdiag_lovl==None:
                    nmom = self.PVDiagMMapSelector.currentIndex()
                    self.mplFigureWidgetPVDiagMMap.canvas.plotMomentMap(imageData=self.imageData, clineVarName=self.LContVarNameMMap, clineVarID=self.LContVarIDMMap, \
                        LContGrid=self.LContGrid, LContGridColor=self.LContGridColor, FContGridColor=self.FContGridColor[self.FContVarNameMMap],\
                        bunit=self.imageData.header['BUNIT'], PVDiagLineOverlay=True, nmom_inp=nmom)

                
                #self.mplFigureWidgetPVDiagMMap.canvas.pvdiag_lovl.set_ydata(y[::-1])
                self.mplFigureWidgetPVDiagMMap.canvas.pvdiag_lovl.set_ydata(y)
                self.mplFigureWidgetPVDiagMMap.canvas.draw()
           
    def updateChannelMapToNewChannel(self):

        if self.imageData.imexists:
            if self.LContVarNameCMap=='Channel Map':
                if self.imageData.image['plotconv']:
                    self.LCdata = self.imageData.image['cdata'][self.imageData.image['stokes_axis'], self.imageData.image['ifreq'],:,:]
                else:
                    self.LCdata = self.imageData.image['data'][self.imageData.image['stokes_axis'],self.imageData.image['ifreq'],:,:]
                

                
            self.ImIndexSlider.setValue(self.imageData.image['ifreq'])
            if self.ImBeamConvCheckBox.isChecked():
                self.mplFigureWidgetCMap.canvas.cm_plot.set_array(self.imageData.image['cdata'][self.imageData.image['stokes_axis'], self.imageData.image['ifreq'],::-1,:])

            else:
                self.mplFigureWidgetCMap.canvas.cm_plot.set_array(self.imageData.image['data'][self.imageData.image['stokes_axis'],self.imageData.image['ifreq'],::-1,:])

            if self.ImCMapCLinesEnableCheckBox.isChecked():
                for i in self.mplFigureWidgetCMap.canvas.cm_lines.collections:
                    if self.mplFigureWidgetCMap.canvas.axes.collections.__contains__(i):
                        self.mplFigureWidgetCMap.canvas.axes.collections.remove(i)

                clev = self.LContGrid[self.LContVarNameCMap].val
                if self.LContGridColor[self.LContVarNameCMap].isCurrentSingleColor:
                    color = self.LContGridColor[self.LContVarNameCMap].currentColor
                    self.mplFigureWidgetCMap.canvas.cm_lines = self.mplFigureWidgetCMap.canvas.axes.contour(self.imageData.image['crd_x'],\
                            np.flipud(self.imageData.image['crd_y']), self.LCdata[::-1,:], clev, colors=color)
                else:
                    cmap= self.LContGridColor[self.LContVarNameCMap].colorMaps[self.LContGridColor[self.LContVarNameCMap].currentColorMap]
                    self.mplFigureWidgetCMap.canvas.cm_lines = self.mplFigureWidgetCMap.canvas.axes.contour(self.imageData.image['crd_x'],\
                            np.flipud(self.imageData.image['crd_y']), self.LCdata[::-1,:], clev, cmap=cmap)

            self.mplFigureWidgetCMap.canvas.fig.canvas.draw()


    def updateChannelMap(self):

        if not self.inOpeningImage:
            if self.imageData.imexists:
                self.readOutCMapParams()
                txt = str(self.imageData.image['crd_vel'][self.imageData.image['ifreq']])
                self.ImVelLedit.setText(txt)
                self.ImIndexSlider.setValue(self.imageData.image['ifreq'])
                self.mplFigureWidgetCMap.canvas.plotChannelMap(imageData=self.imageData,clineVarName=self.LContVarNameCMap, LContGrid=self.LContGrid,\
                        LContGridColor=self.LContGridColor, LCdata=self.LCdata[::-1,:], FContGridColor=self.FContGridColor['Channel Map'])

    def updateMomentMapClips(self):
        if self.imageData.imexists:
            mmapParamReadoutOK = self.readOutMMapParams()
            if not mmapParamReadoutOK : return
            self.imageData.makeMomentMap()
            self.switchConvMMap()
            self.mplFigureWidgetMMap.canvas.plotMomentMap(imageData=self.imageData, clineVarName=self.LContVarNameMMap, clineVarID=self.LContVarIDMMap, \
                    LContGrid=self.LContGrid, LContGridColor=self.LContGridColor, FContGridColor=self.FContGridColor[self.FContVarNameMMap],\
                    bunit=self.imageData.header['BUNIT'])
            

            txt = str(self.imageData.mmap['clipMin'][self.imageData.mmap['nmom']])
            self.ImMMapClipMinLedit.setText(txt)
            txt = str(self.imageData.mmap['clipMax'][self.imageData.mmap['nmom']])
            self.ImMMapClipMaxLedit.setText(txt)

    def updateMomentMap(self):

        if not self.inOpeningImage:
            if self.imageData.imexists:
                mmapParamReadoutOK = self.readOutMMapParams()
                if not mmapParamReadoutOK : return
                self.switchConvMMap()
                self.mplFigureWidgetMMap.canvas.plotMomentMap(imageData=self.imageData, clineVarName=self.LContVarNameMMap, clineVarID=self.LContVarIDMMap, \
                        LContGrid=self.LContGrid, LContGridColor=self.LContGridColor, FContGridColor=self.FContGridColor[self.FContVarNameMMap],\
                        bunit=self.imageData.header['BUNIT'])

                txt = str(self.imageData.mmap['clipMin'][self.imageData.mmap['nmom']])
                self.ImMMapClipMinLedit.setText(txt)
                txt = str(self.imageData.mmap['clipMax'][self.imageData.mmap['nmom']])
                self.ImMMapClipMaxLedit.setText(txt)

    def resetMMapClipMin(self):

        if self.imageData.imexists:
            if self.ImMMapBeamConvCheckBox.isChecked():
                val = self.imageData.mmap['cdata'][:,:,self.imageData.mmap['nmom']].min()
                if self.imageData.mmap['nmom']==2:
                    val = np.sqrt(val)
            else:
                val = self.imageData.mmap['data'][:,:,self.imageData.mmap['nmom']].min()
                if self.imageData.mmap['nmom']==2:
                    val = np.sqrt(val)

            self.ImMMapClipMinLedit.setText(str(val))
            self.imageData.mmap['clipMin'][self.imageData.mmap['nmom']] = val
            if not self.inOpeningImage:
                self.updateMomentMap()

    def resetMMapClipMax(self):

        if self.imageData.imexists:
            if self.ImMMapBeamConvCheckBox.isChecked():
                val = self.imageData.mmap['cdata'][:,:,self.imageData.mmap['nmom']].max()
                if self.imageData.mmap['nmom']==2:
                    val = np.sqrt(val)
            else:
                val = self.imageData.mmap['data'][:,:,self.imageData.mmap['nmom']].max()
                if self.imageData.mmap['nmom']==2:
                    val = np.sqrt(val)
                
            self.ImMMapClipMaxLedit.setText(str(val))
            self.imageData.mmap['clipMax'][self.imageData.mmap['nmom']] = val
            if not self.inOpeningImage:
                self.updateMomentMap()
           
    def readOutMMapParams(self):
        
        if self.imageData.imexists:
            dum = str(self.ImMMapThresholdSelectorLedit.text())
            if dum.strip()=='':
                dum = str(self.imageData.image['data'].min())+', '+str(self.imageData.image['data'].max())
                self.ImMMapThresholdSelectorLedit.setText(dum) 
            try:
                dum = dum.split(',')
            except:
                msg = "Invalid value for 'thresold' value for moment maps. Lower and higher threshold values should be separated by a comma."
                reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
                log.exception(msg)
                return False 


            val1 = str2float(dum[0], 'Threshold (Moment Map)') 
            if val1==None: return False
            val2 = str2float(dum[1], 'Threshold (Moment Map)') 
            if val2==None: return False
            
            if self.imageData.mmap['threshold'][0]!=val1:
                self.imageData.mmap['threshold'][0]=val1

            if self.imageData.mmap['threshold'][1]!=val2:
                self.imageData.mmap['threshold'][1]=val2


            dum = str(self.ImMMapChannelSelectorLedit.text())
            if dum.strip()=='':
                dum = '0, '+str(self.imageData.image['crd_vel'].shape[0]-1)
                self.ImMMapChannelSelectorLedit.setText(dum) 
            try:
                dum = dum.split(',')
            except:
                msg = "Invalid value for 'channels' value for moment maps. Values should be separated by a comma."
                reply = QtGui.QMessageBox.critical(None,"Error", msg, QtGui.QMessageBox.Ok)
                log.exception(msg)
                return False

            val1 = str2int(dum[0], 'Channels (Moment Map)') 
            if val1==None: return False
            val2 = str2int(dum[1], 'Channels (Moment Map)') 
            if val2==None: return False
            
            if self.imageData.mmap['channel_ind'][0]!=val1:
                self.imageData.mmap['channel_ind'][0] = val1
            if self.imageData.mmap['channel_ind'][1]!=val2:
                self.imageData.mmap['channel_ind'][1] = val2

            self.imageData.mmap['enableCLines'] = self.ImMMapCLinesEnableCheckBox.isChecked()
           
            if self.ImMMapScaleLinRButton.isChecked():self.imageData.mmap['logscale']=False
            if self.ImMMapScaleLogRButton.isChecked():self.imageData.mmap['logscale']=True

            if self.ImMMapMoment0RButton.isChecked():nmom=0
            if self.ImMMapMoment1RButton.isChecked():nmom=1
            if self.ImMMapMoment2RButton.isChecked():nmom=2

            self.LContVarNameMMap = str(self.ImMMapCLinesMapSelectorCBox.currentText())
            self.LContVarIDMMap = self.ImContourVarName[1:].index(self.LContVarNameMMap) 


            if (self.imageData.mmap['nmom']!=nmom):
                self.imageData.mmap['nmom'] = nmom
                self.imageData.mmap['mapchanged'] = True

                self.FContVarNameMMap = self.ImContourVarName[nmom+1]
                cmapid = self.FContGridColor[self.FContVarNameMMap].colorMaps.keys().index(\
                        self.FContGridColor[self.FContVarNameMMap].currentColorMap)
                self.ImMMapColorMapSelectorCBox.setCurrentIndex(cmapid)

                self.inOpeningImage = True
                self.resetMMapClipMin()
                self.resetMMapClipMax()



                txt = str(self.imageData.mmap['clipMin'][self.imageData.mmap['nmom']])
                self.ImMMapClipMinLedit.setText(txt)
                txt = str(self.imageData.mmap['clipMax'][self.imageData.mmap['nmom']])
                self.ImMMapClipMaxLedit.setText(txt)
                self.inOpeningImage = False
            else:
                txt = str(self.ImMMapClipMinLedit.text())
                if txt.strip()=='':
                    txt = '-1e99'
                    self.ImMMapClipMinLedit.setText(txt)
                self.imageData.mmap['clipMin'][self.imageData.mmap['nmom']] = str2float(txt, 'Clip Min. (Moment Map)')
                txt = str(self.ImMMapClipMaxLedit.text())
                if txt.strip()=='':
                    txt = '-1e99'
                    self.ImMMapClipMaxLedit.setText(txt)
                self.imageData.mmap['clipMax'][self.imageData.mmap['nmom']] = str2float(txt, 'Clip Max. (Moment Map)')
          
                if nmom==0:
                    self.FContGridColor['0th Moment Map'].currentColorMap = str(self.ImMMapColorMapSelectorCBox.currentText())  
                    self.FContVarNameMMap = '0th Moment Map'
                if nmom==1:
                    self.FContGridColor['1st Moment Map'].currentColorMap = str(self.ImMMapColorMapSelectorCBox.currentText())  
                    self.FContVarNameMMap = '1st Moment Map'
                if nmom==2:
                    self.FContGridColor['2nd Moment Map'].currentColorMap = str(self.ImMMapColorMapSelectorCBox.currentText())  
                    self.FContVarNameMMap = '2nd Moment Map'
        return True
    
    def setMMapParams(self):
        
        if self.imageData.imexists:
            if self.imageData.mmap['enableCLines']:
                self.ImMMapCLinesEnableCheckBox.setChecked()

            txt = str(self.imageData.mmap['clipMin'][self.imageData.mmap['nmom']])
            self.ImMMapClipMinLedit.setText(txt)
            txt = str(self.imageData.mmap['clipMax'][self.imageData.mmap['nmom']])
            self.ImMMapClipMaxLedit.setText(txt)

            self.ImMMapThresholdSelectorLedit.setText(str(self.imageData.mmap['threshold'][0])+','+str(self.imageData.mmap['threshold'][1]))
            self.ImMMapChannelSelectorLedit.setText(str(self.imageData.mmap['channel_ind'][0])+','+str(self.imageData.mmap['channel_ind'][1]))


    def switchConvMMap(self):
        
        if self.imageData.imexists:
            if self.ImMMapBeamConvCheckBox.isChecked():
                self.imageData.mmap['plotconv'] = True

                do_conv = False
                if self.imageData.mmap['mapchanged']:
                    do_conv = True
                    self.imageData.mmap['mapchanged']=False

                if (len(self.imageData.mmap['cdata'].shape)<2): do_conv = True

                val = str2float(self.ImMMapBmajLedit.text(), 'Major axis (Moment maps)')
                if (val!=self.imageData.mmap['bmaj']): 
                    do_conv = True
                    self.imageData.mmap['bmaj'] = val

                val = str2float(self.ImMMapBminLedit.text(), 'Minor axis (Moment maps)')
                if (val!=self.imageData.mmap['bmin']): 
                    do_conv = True
                    self.imageData.mmap['bmin'] = val

                val = str2float(self.ImMMapPALedit.text(), 'Position angle (Moment maps)')
                if (val!=self.imageData.mmap['pa']): 
                    do_conv = True
                    self.imageData.mmap['pa'] = val

                nmom = -1
                if self.ImMMapMoment0RButton.isChecked():nmom=0
                if self.ImMMapMoment1RButton.isChecked():nmom=1
                if self.ImMMapMoment2RButton.isChecked():nmom=2
                if self.imageData.mmap['nmom']!=nmom: do_con = True

                if do_conv:
                    self.imageData.image['bmaj'] = self.imageData.mmap['bmaj']
                    self.imageData.image['bmin'] = self.imageData.mmap['bmin']
                    self.imageData.image['pa'] = self.imageData.mmap['pa']
                    self.imageData.imconv()
                    self.imageData.makeMomentMap(conv=True)
                    self.ImBmajLedit.setText(str(self.ImMMapBmajLedit.text()))
                    self.ImBminLedit.setText(str(self.ImMMapBminLedit.text()))
                    self.ImPALedit.setText(str(self.ImMMapPALedit.text()))
            
            else:
                self.imageData.mmap['plotconv'] = False
            


    def switchConvPVDiag(self):

        if self.imageData.imexists:
            if self.PVDiagBeamConvCheckBox.isChecked():
                self.imageData.pvdiag['plotconv'] = True
                do_conv = False

                val = str2float(self.PVDiagBmajLedit.text(), 'Major axis (Moment maps)')
                if (val!=self.imageData.pvdiag['bmaj']): 
                    do_conv = True
                    self.imageData.pvdiag['bmaj'] = val

                val = str2float(self.PVDiagBminLedit.text(), 'Minor axis (Moment maps)')
                if (val!=self.imageData.pvdiag['bmin']): 
                    do_conv = True
                    self.imageData.pvdiag['bmin'] = val

                val = str2float(self.PVDiagBPALedit.text(), 'Position angle (Moment maps)')
                if (val!=self.imageData.pvdiag['pa']): 
                    do_conv = True
                    self.imageData.pvdiag['pa'] = val

                if do_conv:
                    self.imageData.image['bmaj'] = self.imageData.pvdiag['bmaj']
                    self.imageData.image['bmin'] = self.imageData.pvdiag['bmin']
                    self.imageData.image['pa'] = self.imageData.pvdiag['pa']
                    self.imageData.imconv()
                    self.imageData.makePVDiagram(conv=True)
                    self.ImBmajLedit.setText(str(self.PVDiagBmajLedit.text()))
                    self.ImBminLedit.setText(str(self.PVDiagBminLedit.text()))
                    self.ImPALedit.setText(str(self.PVDiagPALedit.text()))
                    print 'Making pv diagram' 
            else:
                self.imageData.pvdiag['plotconv'] = False
            
            self.updatePVDiagram()

