################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
from PyQt4 import QtGui, QtCore
import matplotlib
from matplotlib import cm, ticker
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.pylab import colorbar, contourf, getp, setp
from matplotlib.colors import Normalize, LogNorm
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from artist_gui.getplotmodel import *
from artist_gui.grid import *
from artist_gui.helper_functions import ConfigurationParams

import sys
import time

matplotlib.rcParams['axes.unicode_minus'] = False

class MplCanvas(FigureCanvas):
    __pyqtSignals__ = ("update2DContours()")
    def __init__(self):

        ConfigParams = ConfigurationParams()
        self.figureConfigParams = ConfigParams.figureParams
        self.cbar = None

        fsizex = 5.0
        fsizey = 5.0
        self.fig = Figure(figsize=(fsizex,fsizey), dpi=100)
        self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True)

        self.axes.hold(True)
        self.axes.set_adjustable('box')
       
        
        FigureCanvas.__init__(self, self.fig)
        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)


        self.PlottedFContVarName  = 'velocity_y'
        self.PlottedLContVarName  = ['density','temperature','velocity_x', 'velocity_y', 'velocity_z', 'abundance', 'tdust', \
                'doppler', 'bmag_x', 'bmag_y', 'bmag_z', 'column_density_x', 'column_density_y', 'column_density_z']
        
        
        self.PlottedFContVarLabel = {'density':'Gas density', 'temperature':'Gas temperature', 'velocity_x':'Velocity (x)',\
                'velocity_y':'Velocity (y)','velocity_z':'Velocity (z)', 'abundance':'Abundance', 'tdust':'Dust Temperature', 'doppler':'Doppler b', \
                'bmag_x':'B (x)', 'bmag_y':'B (y)', 'bmag_z':'B (z)', 'column_density_x':'Gas column density (x)', 'column_density_y':'Gas column density (y)',\
                'column_density_z':'Gas column density (z)'}

        self.PlottedFContVarCGSUnit  = {'density':'[1/cm'+r'$^3$'+']', 'temperature':'[K]', 'velocity_x':'[cm/s]', 'velocity_y':'[cm/s]', \
                'velocity_z':'[cm/s]', 'abundance':'', 'tdust':'[K]','doppler':'[cm/s]','bmag_x':'[G]', 'bmag_y':'[G]', 'bmag_z':'[G]',\
                'column_density_x':r'[1/cm$^2$]', 'column_density_y':r'[1/cm$^2$]', 'column_density_z':r'[1/cm$^2$]'}
        
        self.PlottedFContVarSIUnit  = {'density':'[1/m'+r'$^3$'+']', 'temperature':'[K]', 'velocity_x':'[m/s]', 'velocity_y':'[m/s]', \
                'velocity_z':'[m/s]', 'abundance':'', 'tdust':'[K]','doppler':'[m/s]','bmag_x':'[T]', 'bmag_y':'[T]', 'bmag_z':'[T]',\
                'column_density_x':r'[1/m$^2$]', 'column_density_y':r'[1/m$^2$]', 'column_density_z':r'[1/m$^2$]'}

        self.PlottedLContVar      = {'density':False, 'temperature':False, 'velocity_x':False, 'velocity_y':False, 'velocity_z':False, 'abundance':False,\
                'tdust':False, 'doppler':False, 'bmag_x':False, 'bmag_y':False, 'bmag_z':False, 'column_density_x':False, 'column_density_y':False, \
                'column_density_z':False}

        self.PlottedModelID       = 0
        self.GridPar              = PlotGridParams()
        self.CPlotPar             = PlotContourParams()

        self.var                 = np.zeros((self.GridPar.nx,self.GridPar.ny,self.GridPar.nz,len(self.PlottedLContVarName)),dtype=np.float64)
        self.x                   = np.zeros((self.GridPar.nx),dtype=np.float64)
        self.y                   = np.zeros((self.GridPar.ny),dtype=np.float64)
        self.z                   = np.zeros((self.GridPar.nz),dtype=np.float64)

    def mouseDoubleClickEvent(self,event):
        self.emit(QtCore.SIGNAL('update2DContours()'))

    def updateModelVars(self,MainWindow,defaultPar=False):
        
        self.var                 = np.zeros((self.GridPar.nx,self.GridPar.ny,self.GridPar.nz,len(self.PlottedLContVarName)),dtype=np.float64)
        self.x                   = np.zeros((self.GridPar.nx),dtype=np.float64)
        self.y                   = np.zeros((self.GridPar.ny),dtype=np.float64)
        self.z                   = np.zeros((self.GridPar.nz),dtype=np.float64)
        
        self.getAllModelVars(MainWindow,defaultPar=defaultPar)

    def getAllModelVars(self,MainWindow=None,defaultPar=False): 

        if (MainWindow!=None):
            ivar = 0
            for ivar in range(len(self.PlottedLContVarName)):
                dum = getPlotModelVar(self.GridPar,self.PlottedModelID,self.PlottedLContVarName[ivar],defaultPar=defaultPar,\
                        UnitSystem=MainWindow.parent().configParams.unitSystem)
                self.var[:,:,:,ivar] = dum['var'] 
        else:
            for ivar in range(len(self.PlottedLContVarName)):
                dum = getPlotModelVar(self.GridPar,self.PlottedModelID,self.PlottedLContVarName[ivar],defaultPar=defaultPar,\
                        UnitSystem=MainWindow.parent().configParams.unitSystem)
                self.var[:,:,:,ivar] = dum['var']

        self.x = dum['x']
        self.y = dum['y']
        self.z = dum['z']         

    def plotModelVariableFilledContours(self,Figure, logscale=False, UnitSystem=''):
       
        self.axes.clear()
        self.axes.figure.clf()
       
        fsizex = 5.
        fsizey = 5.
        self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True)
        self.axes.clear()
        nc   = natconst()



        if (self.GridPar.CSectionPlane==0):
            plotVar = self.var[:,0,:,:]
            plotX   = self.x
            plotY   = self.z
            xtitle  = 'x'
            ytitle  = 'z'
        if (self.GridPar.CSectionPlane==1):
            plotVar = self.var[0,:,:,:]
            plotX   = self.y
            plotY   = self.z
            xtitle  = 'y'
            ytitle  = 'z'
        if (self.GridPar.CSectionPlane==2):
            plotVar = self.var[:,:,0,:]
            plotX   = self.x
            plotY   = self.y
            xtitle  = 'x'
            ytitle  = 'y'

        plotX = plotX / self.CPlotPar.plotLimUnit[self.CPlotPar.currentLimUnitID]
        plotY = plotY / self.CPlotPar.plotLimUnit[self.CPlotPar.currentLimUnitID] 
        xtitle = xtitle + ' ['+self.CPlotPar.plotLimUnitName[self.CPlotPar.currentLimUnitID]+ ']'
        ytitle = ytitle + ' ['+self.CPlotPar.plotLimUnitName[self.CPlotPar.currentLimUnitID]+ ']'
        
        ivar = self.PlottedLContVarName.index(self.PlottedFContVarName)
        lev  = self.GridPar.FContGrid[self.PlottedFContVarName].val

        data = np.squeeze(plotVar[:,:,ivar]).transpose()
        cm   = matplotlib.cm.binary
        

        Figure.axes.clear()
        if self.GridPar.FContGrid[self.PlottedFContVarName].gridstyle=='log':

            if ((data.min()==data.max())&(data.min()==0.)):
                data = data.clip(1e-90, 1e+90)

            data = np.log10(data)


            lev = np.log10(lev)
            c = Figure.axes.contourf(plotX, plotY, data, lev, \
                    cmap= self.GridPar.FContGridColor[self.PlottedFContVarName].colorMaps[\
                    self.GridPar.FContGridColor[self.PlottedFContVarName].currentColorMap],\
                    norm=Normalize(vmin=lev.min(), vmax=lev.max(), clip=True))

            self.cbar = self.fig.colorbar(c)
            ticklabels = getp(self.cbar.ax, 'ymajorticklabels')
            txt = [(10.**float(i.get_text())) for i in ticklabels]
            txt = [("%1.2e"%i) for i in txt]
            self.cbar.set_ticklabels(txt)            
            setp(ticklabels, fontsize=self.figureConfigParams['font_size'])
        

        else:
            c = Figure.axes.contourf(plotX, plotY, data, lev, \
                    cmap= self.GridPar.FContGridColor[self.PlottedFContVarName].colorMaps[\
                    self.GridPar.FContGridColor[self.PlottedFContVarName].currentColorMap],\
                    norm=Normalize(vmin=lev.min(), vmax=lev.max(), clip=True))
            
            ticklabels = getp(c.ax, 'ymajorticklabels')
            setp(ticklabels, fontsize=10)

            
            self.cbar = self.fig.colorbar(c)
            ticklabels = getp(self.cbar.ax, 'ymajorticklabels')
            txt = [float(i.get_text().strip()) for i in ticklabels]
            offset     = self.cbar.ax.get_yaxis().get_major_formatter().get_offset()
            if offset!='': 
                txt = [("%1.2e"%(i*float(offset))) for i in txt]
            else:
                txt = [("%1.2e"%i) for i in txt]

            self.cbar.set_ticklabels(txt)            
            setp(ticklabels, fontsize=self.figureConfigParams['font_size'])

        if UnitSystem=='SI':
            if self.PlottedFContVarLabel[self.PlottedFContVarName]=='density':
                self.cbar.ax.set_ylabel('number '+self.PlottedFContVarLabel[self.PlottedFContVarName] + ' ' +\
                    self.PlottedFContVarSIUnit[self.PlottedFContVarName], fontsize=self.figureConfigParams['font_size'])
            else:
                self.cbar.ax.set_ylabel(self.PlottedFContVarLabel[self.PlottedFContVarName] + ' ' +\
                    self.PlottedFContVarSIUnit[self.PlottedFContVarName], fontsize=self.figureConfigParams['font_size'])

        elif UnitSystem=='CGS':
            if self.PlottedFContVarLabel[self.PlottedFContVarName] =='density':
                self.cbar.ax.set_ylabel('number '+self.PlottedFContVarLabel[self.PlottedFContVarName] + ' ' +\
                    self.PlottedFContVarCGSUnit[self.PlottedFContVarName], fontsize=self.figureConfigParams['font_size'])
            else:
                self.cbar.ax.set_ylabel(self.PlottedFContVarLabel[self.PlottedFContVarName] + ' ' +\
                    self.PlottedFContVarCGSUnit[self.PlottedFContVarName], fontsize=self.figureConfigParams['font_size'])


        yticklabels = getp(c.ax, 'ymajorticklabels')
        xticklabels = getp(c.ax, 'xmajorticklabels')
        setp(yticklabels, fontsize=self.figureConfigParams['font_size'])
        setp(xticklabels, fontsize=self.figureConfigParams['font_size'])
           
        Figure.axes.set_xlim(self.CPlotPar.plotXlim)
        Figure.axes.set_ylim(self.CPlotPar.plotYlim)
        Figure.axes.set_xlabel(xtitle, fontsize=self.figureConfigParams['font_size'])
        Figure.axes.set_ylabel(ytitle, fontsize=self.figureConfigParams['font_size'])

        Figure.draw()  

    def plotModelVariableContourLines(self,Figure):
        
        nc   = natconst()

        if (self.GridPar.CSectionPlane==0):
            plotVar = self.var[:,0,:,:]
            plotX   = self.x
            plotY   = self.z

        if (self.GridPar.CSectionPlane==1):
            plotVar = self.var[0,:,:,:]
            plotX   = self.y
            plotY   = self.z
        if (self.GridPar.CSectionPlane==2):
            plotVar = self.var[:,:,0,:]
            plotX   = self.x
            plotY   = self.y

        plotX = plotX / self.CPlotPar.plotLimUnit[self.CPlotPar.currentLimUnitID]
        plotY = plotY / self.CPlotPar.plotLimUnit[self.CPlotPar.currentLimUnitID] 

        
        for ivar in range(len(self.PlottedLContVarName)):
            if (self.PlottedLContVar[self.PlottedLContVarName[ivar]]==True):
                lev = self.GridPar.LContGrid[self.PlottedLContVarName[ivar]].val
               
                data = plotVar[:,:,ivar].transpose()
                if self.PlottedLContVarName[ivar]=='density':
                    data = np.log10(data)
                    lev  = np.log10(lev)

                if self.GridPar.LContGridColor[self.PlottedLContVarName[ivar]].isCurrentSingleColor:
                    Figure.axes.contour(plotX, plotY, data, lev, colors=self.GridPar.LContGridColor[self.PlottedLContVarName[ivar]].currentColor, alpha=0.5)
                else:
                    Figure.axes.contour(plotX, plotY, data, lev, \
                            cmap= self.GridPar.LContGridColor[self.PlottedLContVarName[ivar]].colorMaps[\
                            self.GridPar.LContGridColor[self.PlottedLContVarName[ivar]].currentColorMap], alpha=0.5)

                Figure.axes.set_xlim(self.CPlotPar.plotXlim)
                Figure.axes.set_ylim(self.CPlotPar.plotYlim)


        Figure.draw()  

    def updateAxisParams(self, figureConfigParams=None):

        self.figureConfigParams = figureConfigParams
      


        self.axes.set_xlabel(self.axes.xaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])
        self.axes.set_ylabel(self.axes.yaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])
        self.axes.tick_params(axis='both', labelsize=self.figureConfigParams['font_size'])
        if self.cbar!=None:
            self.cbar.ax.set_ylabel(self.cbar.ax.yaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])
       
            ticklabels = getp(self.cbar.ax, 'ymajorticklabels') 
            setp(ticklabels, fontsize=self.figureConfigParams['font_size'])

        self.draw()

class PlotGridParams():
    def __init__(self, parent=None):

        nc  = natconst()
        au  = 1.496e11

        self.nx   = 50
        self.xmin = 1.0 * au
        self.xmax = 100. * au
        self.xl   = None
        self.xpos = None
        self.xval = 0.0

        self.ny   = self.nx
        self.ymin = self.xmin
        self.ymax = self.xmax
        self.yl   = None
        self.ypos = None
        self.ycouple2x = True
        self.yval = 0.0

        self.nz   = self.nx
        self.zmin = self.xmin
        self.zmax = self.xmax
        self.zl   = None
        self.zpos = None
        self.zcouple2x = True
        self.zval = 0.0
    
        self.gridChanged = False
        self.modelChanged = False
        self.xylimfromimage = False

        self.CSectionPlane        = 0
        self.CSection3rdAxisValue = 0.0


        self.LContGrid = {}
        self.FContGrid = {}
        self.LContGridColor={} 
        self.FContGridColor={} 
        
        self.LContGridColor['density'] = GridColor()
        self.FContGridColor['density'] = GridColor()
        
        self.LContGridColor['temperature'] = GridColor()
        self.FContGridColor['temperature'] = GridColor()
        
        self.LContGridColor['velocity_x'] = GridColor()
        self.FContGridColor['velocity_x'] = GridColor()
        
        self.LContGridColor['velocity_y'] = GridColor()
        self.FContGridColor['velocity_y'] = GridColor()
        
        self.LContGridColor['velocity_z'] = GridColor()
        self.FContGridColor['velocity_z'] = GridColor()
        
        self.LContGridColor['abundance'] = GridColor()
        self.FContGridColor['abundance'] = GridColor()
        
        self.LContGridColor['tdust'] = GridColor()
        self.FContGridColor['tdust'] = GridColor()
        
        self.LContGridColor['doppler'] = GridColor()
        self.FContGridColor['doppler'] = GridColor()

        self.LContGridColor['bmag_x'] = GridColor()
        self.FContGridColor['bmag_x'] = GridColor()
        
        self.LContGridColor['bmag_y'] = GridColor()
        self.FContGridColor['bmag_y'] = GridColor()
        
        self.LContGridColor['bmag_z'] = GridColor()
        self.FContGridColor['bmag_z'] = GridColor()
        
        self.LContGridColor['column_density_x'] = GridColor()
        self.FContGridColor['column_density_x'] = GridColor()
        
        self.LContGridColor['column_density_y'] = GridColor()
        self.FContGridColor['column_density_y'] = GridColor()
        
        self.LContGridColor['column_density_z'] = GridColor()
        self.FContGridColor['column_density_z'] = GridColor()


        self.LContGrid['density'] = Grid(minval=1e4, maxval=1e15, nval=30, gridstyle='log')
        self.FContGrid['density'] = Grid(minval=1e4, maxval=1e15, nval=30, gridstyle='log')
        
        self.LContGrid['temperature'] = Grid(minval=10., maxval=1500., nval=30, gridstyle='lin')
        self.FContGrid['temperature'] = Grid(minval=10., maxval=1500., nval=30, gridstyle='lin')
        
        self.LContGrid['velocity_x'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        self.FContGrid['velocity_x'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')

        self.LContGrid['velocity_y'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        self.FContGrid['velocity_y'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        
        self.LContGrid['velocity_z'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        self.FContGrid['velocity_z'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        
        self.LContGrid['abundance'] = Grid(minval=1e-9, maxval=1e-4, nval=30, gridstyle='lin')
        self.FContGrid['abundance'] = Grid(minval=1e-9, maxval=1e-4, nval=30, gridstyle='lin')
        
        self.LContGrid['tdust'] = Grid(minval=10., maxval=1500, nval=30, gridstyle='lin')
        self.FContGrid['tdust'] = Grid(minval=10., maxval=1500, nval=30, gridstyle='lin')
        
        self.LContGrid['doppler'] = Grid(minval=1., maxval=1e3, nval=30, gridstyle='lin')
        self.FContGrid['doppler'] = Grid(minval=1., maxval=1e3, nval=30, gridstyle='lin')
        
        self.LContGrid['bmag_x'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        self.FContGrid['bmag_x'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')

        self.LContGrid['bmag_y'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        self.FContGrid['bmag_y'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        
        self.LContGrid['bmag_z'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
        self.FContGrid['bmag_z'] = Grid(minval=-1e5, maxval=1e5, nval=30, gridstyle='lin')
   
        self.LContGrid['column_density_x'] = Grid(minval=-1e5, maxval=1e25, nval=30, gridstyle='lin')
        self.FContGrid['column_density_x'] = Grid(minval=-1e5, maxval=1e25, nval=30, gridstyle='lin')

        self.LContGrid['column_density_y'] = Grid(minval=-1e5, maxval=1e25, nval=30, gridstyle='lin')
        self.FContGrid['column_density_y'] = Grid(minval=-1e5, maxval=1e25, nval=30, gridstyle='lin')
        
        self.LContGrid['column_density_z'] = Grid(minval=-1e5, maxval=1e25, nval=30, gridstyle='lin')
        self.FContGrid['column_density_z'] = Grid(minval=-1e5, maxval=1e25, nval=30, gridstyle='lin')

class PlotContourParams():
    def __init__(self, parent=None):
       
        self.plotXlim = [-100., 100.]
        self.plotYlim = [-100., 100.]
        self.FContNum = 50
        self.LContNum = 50
        self.plotLimUnitName = ['cm', 'm', 'AU', 'pc']
        self.plotLimUnit = (1e-2, 1.0, 1.496e11, 3.0857200e+16)
        self.currentLimUnitID = 2

class mplFigure2D(QtGui.QWidget):
    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)


        self.canvas = MplCanvas()
        self.toolbar = NavigationToolbar(self.canvas, self)
        self.vbl = QtGui.QVBoxLayout()
        self.vbl.addWidget(self.canvas)
        self.vbl.addWidget(self.toolbar)
        self.setLayout(self.vbl)
        


