################################################################################
##
## Copyright 2012, Attila Juhasz,
## Leiden Observatory
## Leiden Univeristy
## juhasz@strw.leidenuniv.nl 
##
## and the ARTIST team.
##
## For further info about copyright and license see included license.txt file.
##
################################################################################


import numpy as np
from PyQt4 import QtGui, QtCore
import matplotlib
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.pylab import colorbar, contourf, imshow, getp, setp, cm
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.widgets import Cursor
from matplotlib.patches import Rectangle
from artist_gui.helper_functions import ConfigurationParams
import sys

import time

class MyToolbar(NavigationToolbar):
   
    writePVDiagramToFits = QtCore.pyqtSignal(object,name='writePVDiagramToFits') 

    def save_figure(self):
        filters = ['Encapsulated postscript (*.eps)', 'Portable Document Format (*.pdf)', 'Portable Network Graphics (*.png)', 'Postscript (*.ps)', 'FITS (*.fits)']
        filters_txt = filters[0]
        for ifilt in filters[1:]:
            filters_txt = filters_txt+';;'+ifilt

        fname, filterName =  QtGui.QFileDialog.getSaveFileNameAndFilter(self, 'Choose a filename to save the image to', \
                filter=filters_txt, options=QtGui.QFileDialog.DontUseNativeDialog, initialFilter=filters[1])
        fname = str(fname).strip()
        filterName = str(filterName).strip()

        filter_id = -1
        if filterName==filters[0]:
            if fname[len(fname.strip())-4:]!='.eps':
                fname=fname+'.eps'
            filter_id = 0
            
        if filterName==filters[1]:
            if fname[len(fname.strip())-4:]!='.pdf':
                fname=fname+'.pdf'
            filter_id = 1
        
        if filterName==filters[2]:
            if fname[len(fname.strip())-4:]!='.png':
                fname=fname+'.png'
            filter_id = 2
            
        if filterName==filters[3]:
            if fname[len(fname.strip())-3:]!='.ps':
                fname=fname+'.ps'
            filter_id = 3
            
        if filterName==filters[4]:
            if fname[len(fname.strip())-5:]!='.fits':
                fname=fname+'.fits'
            filter_id = 4

        if filter_id>=0:
            if filter_id<4:
                self.parent().canvas.fig.savefig(fname, facecolor=self.parent().canvas.figureConfigParams['bgcolor'])
            if filter_id==4:
                self.writePVDiagramToFits.emit(fname)

class MplCanvasImage(FigureCanvas):
    
    __pyqtSignals__ = ("ClickedOnCanvas(float, float, int)", 'float', 'float', 'int')


    def __init__(self):
        

        ConfigParams = ConfigurationParams()
        self.figureConfigParams = ConfigParams.figureParams


        fsizex = 5.0
        fsizey = 5.0
        self.fig = Figure(figsize=(fsizex,fsizey), dpi=100, facecolor=self.figureConfigParams['bgcolor'])
        self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True, axisbg=self.figureConfigParams['bgcolor'])


        self.axes.spines['bottom'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['top'].set_color(self.figureConfigParams['fgcolor']) 
        self.axes.spines['right'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['left'].set_color(self.figureConfigParams['fgcolor'])
    
        self.axes.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.axes.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 

        self.axes.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.axes.yaxis.label.set_color(self.figureConfigParams['fgcolor'])

        self.axes.hold(True)
        self.axes.set_adjustable('box')

        self.cbar = None
       
        
        FigureCanvas.__init__(self, self.fig)
        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        self.enablePixSpec = False 

        self.selection_p0 = []
        self.selection_p1 = []
        self.selection_x  = []
        self.selection_y  = []
        self.selection_rect = None
        self.pvdiag_lovl = None

        cid1 = self.mpl_connect('button_press_event', self.singleClick)

    def mouseMoved(self, event):

        if event.button==1:
            
            self.selection_p1 = np.array([event.xdata, event.ydata])
            if self.selection_rect==None:
                self.selection_rect = Rectangle( self.selection_p0, self.selection_p1[0]-self.selection_p0[0], self.selection_p1[1]-self.selection_p0[1], facecolor='none',\
                    edgecolor='white', linewidth=0.5)
            else:
                self.selection_rect.set_bounds(self.selection_p0[0], self.selection_p0[1], \
                         self.selection_p1[0]-self.selection_p0[0], self.selection_p1[1]-self.selection_p0[1])
            self.axes.add_patch(self.selection_rect)
            self.draw()

    def singleClick(self,event):
        if event.xdata:
            if event.ydata:
                self.selection_p0=np.array([event.xdata, event.ydata])
                axes_bounds = event.inaxes.get_window_extent().bounds
                if axes_bounds[3]/axes_bounds[2]<10.:
                    self.emit(QtCore.SIGNAL("ClickedOnCanvas(float, float, int)"), event.xdata, event.ydata, event.button)

    def plotPVDiagram(self, imageData=None, clineVarName=None, LContGrid=None, LContGridColor=None, FContGridColor=None):
        
        self.fig.clf()
        fsizex = 5.0
        fsizey = 5.0

        self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True, axisbg=self.figureConfigParams['bgcolor'])

        self.axes.clear()
      
        self.axes.spines['bottom'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['top'].set_color(self.figureConfigParams['fgcolor']) 
        self.axes.spines['right'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['left'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.axes.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 
        self.axes.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.axes.yaxis.label.set_color(self.figureConfigParams['fgcolor'])
        
        self.axes.tick_params(axis='x', labelsize=self.figureConfigParams['font_size'])
        self.axes.tick_params(axis='y', labelsize=self.figureConfigParams['font_size'])
        self.axes.hold(True)
        self.axes.set_aspect('auto', 'datalim')

        if imageData.pvdiag['plotconv']:
            data = imageData.pvdiag['cdata'][:,imageData.pvdiag['posind']-1,:]
        else:
            data = imageData.pvdiag['data'][:,imageData.pvdiag['posind']-1,:]

        data = data.clip(imageData.pvdiag['clipMin'], imageData.pvdiag['clipMax'])
        
        if imageData.pvdiag['logscale']: 
            data = np.log10(data)
            if ((imageData.header['BUNIT']=='K')|(imageData.header['BUNIT']=='KELVIN')):
                cbar_label = '[log(K)]'
            if (imageData.header['BUNIT']=='JY/PIXEL'):
                cbar_label = '[log(Jy/pixel)]'
            if (imageData.header['BUNIT']=='JY/BEAM'):
                cbar_label = '[log(Jy/beam)]'

        else:
            if ((imageData.header['BUNIT']=='K')|(imageData.header['BUNIT']=='KELVIN')):
                cbar_label = '[K]'

            elif (imageData.header['BUNIT']=='JY/PIXEL'):
                cbar_label = '[Jy/pixel]'
            
            elif (imageData.header['BUNIT']=='JY/BEAM'):
                cbar_label = '[Jy/beam]'

            else:
                cbar_label = imageData.header['BUNIT']

        v      = imageData.pvdiag['crd_vel'] 
        velid1 = 0
        velid2 = v.shape[0]-1
        r      = imageData.pvdiag['crd_x']
        if imageData.pvdiag['crd_y'].max()>imageData.pvdiag['crd_x'].max():
            r = imageData.pvdiag['crd_y']
        
        xid1   = 0
        xid2   = r.shape[0]-1

        if imageData.image['crd_vel'][0]<imageData.image['crd_vel'][1]:
            if imageData.pvdiag['logscale']:
                self.pvd_plot = self.axes.imshow(data[:,::-1], interpolation='nearest', cmap=FContGridColor.colorMaps[FContGridColor.currentColorMap], \
                        norm=matplotlib.colors.Normalize(vmin=np.log10(imageData.pvdiag['clipMin']), vmax=np.log10(imageData.pvdiag['clipMax'])), \
                        extent=(r[xid2], r[xid1], v[velid1], v[velid2]))

            else:
                self.pvd_plot = self.axes.imshow(data[:,::-1], interpolation='nearest', cmap=FContGridColor.colorMaps[FContGridColor.currentColorMap], \
                        norm=matplotlib.colors.Normalize(vmin=imageData.pvdiag['clipMin'], vmax=imageData.pvdiag['clipMax']), \
                        extent=(r[xid2], r[xid1], v[velid1], v[velid2]))
        else:
            if imageData.pvdiag['logscale']:
                self.pvd_plot = self.axes.imshow(data[:,::-1], interpolation='nearest', cmap=FContGridColor.colorMaps[FContGridColor.currentColorMap], \
                        norm=matplotlib.colors.Normalize(vmin=np.log10(imageData.pvdiag['clipMin']), vmax=np.log10(imageData.pvdiag['clipMax'])), \
                        extent=(r[xid2], r[xid1], v[velid2], v[velid1]))
            else:
                self.pvd_plot = self.axes.imshow(data[:,::-1], interpolation='nearest', cmap=FContGridColor.colorMaps[FContGridColor.currentColorMap], \
                        norm=matplotlib.colors.Normalize(vmin=imageData.pvdiag['clipMin'], vmax=imageData.pvdiag['clipMax']), \
                        extent=(r[xid2], r[xid1], v[velid2], v[velid1]))
        
        dum = self.axes.get_images()
        extent = dum[0].get_extent()
        aspect = float(data.shape[1])/float(data.shape[0])
        new_aspratio = abs((extent[1]-extent[0])/(extent[3]-extent[2]))/aspect
        self.axes.set_aspect(new_aspratio)
        self.axes.set_xlabel('Offset [arcsec]')
        self.axes.set_ylabel('Velocity [km/s]')


        self.cbar = self.fig.colorbar(self.pvd_plot)
        ticklabels = getp(self.cbar.ax, 'ymajorticklabels') 
        setp(ticklabels, fontsize=self.figureConfigParams['colorbar_fontsize'])
        self.cbar.ax.set_ylabel(cbar_label)
  
        self.cbar.outline.set_color(self.figureConfigParams['fgcolor'])
        self.cbar.ax.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.cbar.ax.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 
        self.cbar.ax.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.cbar.ax.yaxis.label.set_color(self.figureConfigParams['fgcolor'])

        self.cbar.ax.tick_params(length=3.5, width=1.0, left='on', right='on')

        self.cbar.ax.hold(True)
        self.cbar.ax.set_adjustable('box')
       
        self.draw() 

    def plotMomentMap(self, imageData=None, clineVarName=None, clineVarID=None, LContGrid=None, LContGridColor=None, FContGridColor=None, \
            bunit=None, PVDiagLineOverlay=False, nmom_inp=None):

        self.fig.clf()
        
        if PVDiagLineOverlay:
            fsizex = 4.
            fsizey = 4.
            self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True, axisbg=self.figureConfigParams['bgcolor'])

        else:
            fsizex = 5.0
            fsizey = 5.0
            self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True, axisbg=self.figureConfigParams['bgcolor'])

        self.axes.clear()
       
        if PVDiagLineOverlay:
            nmom = nmom_inp
        else:
            nmom = imageData.mmap['nmom']

        self.axes.spines['bottom'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['top'].set_color(self.figureConfigParams['fgcolor']) 
        self.axes.spines['right'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['left'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.axes.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 
        self.axes.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.axes.yaxis.label.set_color(self.figureConfigParams['fgcolor'])
       
        self.axes.tick_params(axis='x', labelsize=self.figureConfigParams['font_size'])
        self.axes.tick_params(axis='y', labelsize=self.figureConfigParams['font_size'])

        if imageData.mmap['plotconv']:
           data = imageData.mmap['cdata']
        else:
           data = imageData.mmap['data']

        cdata = np.array(data)[::-1,::-1]

        if nmom==0:
            if bunit==None:
                cbar_label='Unknown unit'
            else:
                cbar_label = '['+bunit+' km/s]'
        if nmom==1:
            cbar_label='[km/s]'
        if nmom==2:
            cbar_label='[km/s]'
            data = np.sqrt(data.clip(0.,1e30))
       
        if imageData.mmap['logscale']: 
           data = np.log10(data)
           cbar_label='log('+cbar_label+')'
       
        cmap =FContGridColor.colorMaps[FContGridColor.currentColorMap]
        cmap.set_bad('k', 1.0)
        self.mmapPlot = self.axes.imshow(data[::-1,:,nmom].clip(imageData.mmap['clipMin'][nmom], imageData.mmap['clipMax'][nmom]), \
                extent=(imageData.mmap['crd_x'][0], imageData.mmap['crd_x'][imageData.mmap['crd_x'].shape[0]-1], \
                imageData.mmap['crd_y'][0], imageData.mmap['crd_y'][imageData.mmap['crd_y'].shape[0]-1]), \
                interpolation='nearest', cmap=cmap)


        self.axes.set_xlabel('RA offset [arcsec]')
        self.axes.set_ylabel('DEC offset [arcsec]')


        if PVDiagLineOverlay:
             self.axes.tick_params(axis='both', which='major', labelsize=self.figureConfigParams['font_size'])
             self.axes.xaxis.label.set_size(self.figureConfigParams['font_size'])
             self.axes.yaxis.label.set_size(self.figureConfigParams['font_size'])

        if imageData.mmap['enableCLines']:
            clev = LContGrid[clineVarName].val
            if LContGridColor[clineVarName].isCurrentSingleColor:
               color = LContGridColor[clineVarName].currentColor
               self.axes.contour(imageData.mmap['crd_x'][::-1], np.flipud(imageData.mmap['crd_y']), cdata[:,:,clineVarID], clev, colors=color)
            else:
               cmap= LContGridColor[clineVarName].colorMaps[LContGridColor[clineVarName].currentColorMap]
               self.axes.contour(imageData.mmap['crd_x'][::-1], np.flipud(imageData.mmap['crd_y']), cdata[:,:,clineVarID], clev, cmap=cmap)

        if PVDiagLineOverlay:
             x = imageData.mmap['crd_x'][::-1]
             ixc = (imageData.mmap['crd_x'].max() + imageData.mmap['crd_x'].min()) * 0.5
             iyc = (imageData.mmap['crd_y'].max() + imageData.mmap['crd_y'].min()) * 0.5 

             r      = imageData.pvdiag['crd_x']
             if imageData.pvdiag['crd_y'].max()>imageData.pvdiag['crd_x'].max():
                r = imageData.pvdiag['crd_y']

             y = (x-ixc) * np.tan(imageData.pvdiag['posang']/180.*np.pi) + ixc + \
                     (r[imageData.pvdiag['posind']-1] - ixc) / np.cos(imageData.pvdiag['posang']/180.*np.pi)

             #self.pvdiag_lovl = self.axes.plot(x,y[::-1],color='w')[0]
             self.pvdiag_lovl = self.axes.plot(x,y,color='w')[0]

        self.axes.set_xlim(imageData.mmap['crd_x'][0], imageData.mmap['crd_x'][imageData.mmap['crd_x'].shape[0]-1])
        self.axes.set_ylim(imageData.mmap['crd_y'][0], imageData.mmap['crd_y'][imageData.mmap['crd_y'].shape[0]-1])
        #self.axes.set_ylim(imageData.mmap['crd_y'].min(), imageData.mmap['crd_y'].max())
        self.cbar = self.fig.colorbar(self.mmapPlot)
        self.cbar.ax.set_ylabel(cbar_label)         
        ticklabels = getp(self.cbar.ax, 'ymajorticklabels') 
        setp(ticklabels, fontsize=self.figureConfigParams['colorbar_fontsize'])
       
        self.cbar.outline.set_color(self.figureConfigParams['fgcolor'])
        self.cbar.ax.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.cbar.ax.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 
        self.cbar.ax.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.cbar.ax.yaxis.label.set_color(self.figureConfigParams['fgcolor'])

        if PVDiagLineOverlay:
           self.cbar.ax.yaxis.label.set_size(10.)
   
        self.cbar.ax.tick_params(length=3.5, width=1.0, left='on', right='on')

        self.draw() 
        return

    def updateAxisParams(self, figureConfigParams=None):

        self.figureConfigParams = figureConfigParams
      

        self.fig.set_facecolor(self.figureConfigParams['bgcolor'])
        self.axes.set_axis_bgcolor(self.figureConfigParams['bgcolor'])

        self.axes.spines['bottom'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['top'].set_color(self.figureConfigParams['fgcolor']) 
        self.axes.spines['right'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['left'].set_color(self.figureConfigParams['fgcolor'])
    
        self.axes.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.axes.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 

        self.axes.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.axes.yaxis.label.set_color(self.figureConfigParams['fgcolor'])


        self.axes.set_xlabel(self.axes.xaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])
        self.axes.set_ylabel(self.axes.yaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])

        self.axes.tick_params(axis='x', labelsize=self.figureConfigParams['font_size'])
        self.axes.tick_params(axis='y', labelsize=self.figureConfigParams['font_size'])

        if self.cbar!=None:
            self.cbar.outline.set_color(self.figureConfigParams['fgcolor'])
            self.cbar.ax.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
            self.cbar.ax.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 
            self.cbar.ax.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
            self.cbar.ax.yaxis.label.set_color(self.figureConfigParams['fgcolor'])
            self.cbar.ax.set_ylabel(self.axes.yaxis.get_label_text(), fontsize=self.figureConfigParams['font_size'])
       
            ticklabels = getp(self.cbar.ax, 'ymajorticklabels') 
            setp(ticklabels, fontsize=self.figureConfigParams['font_size'])

        self.draw()

    def plotChannelMap(self, imageData=None, clineVarName=None, LContGrid=None, LContGridColor=None, LCdata=None, FContGridColor=None):
        
        self.fig.clf()
        fsizex = 5.0
        fsizey = 5.0

        self.axes = self.fig.add_axes([0.1*5./fsizex, 0.1*5./fsizex, 1.-0.15*5./fsizex, 
                                       1.-0.15*5./fsizex], frameon=True)
        self.axes.clear()
      
        self.axes.spines['bottom'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['top'].set_color(self.figureConfigParams['fgcolor']) 
        self.axes.spines['right'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.spines['left'].set_color(self.figureConfigParams['fgcolor'])
        self.axes.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.axes.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 
        self.axes.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.axes.yaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.axes.hold(True)
        self.axes.set_adjustable('box')
      
        self.axes.tick_params(axis='x', labelsize=self.figureConfigParams['font_size'])
        self.axes.tick_params(axis='y', labelsize=self.figureConfigParams['font_size'])

        if imageData.image['plotconv']:
            data = imageData.image['cdata'][imageData.image['stokes_index'],imageData.image['ifreq'],::-1,:]
        else:
            data = imageData.image['data'][imageData.image['stokes_index'],imageData.image['ifreq'],::-1,:]



        data = data.clip(imageData.image['clipMin'], imageData.image['clipMax'])
        
        if imageData.image['logscale']: 
            data = np.log10(data)
            if ((imageData.header['BUNIT']=='K')|(imageData.header['BUNIT']=='KELVIN')):
                cbar_label = '[log(K)]'
            if (imageData.header['BUNIT']=='JY/PIXEL'):
                cbar_label = '[log(Jy/pixel)]'
        else:
            if ((imageData.header['BUNIT']=='K')|(imageData.header['BUNIT']=='KELVIN')):
                cbar_label = '[K]'

            elif (imageData.header['BUNIT']=='JY/PIXEL'):
                cbar_label = 'Jy/pixel]'
            
            elif (imageData.header['BUNIT']=='JY/BEAM'):
                cbar_label = '[Jy/beam]'

            else:
                cbar_label = imageData.header['BUNIT']
        self.cm_plot = self.axes.imshow(data, extent=(imageData.image['crd_x'][0], imageData.image['crd_x'][imageData.image['crd_x'].shape[0]-1], \
                imageData.image['crd_y'][0], imageData.image['crd_y'][imageData.image['crd_y'].shape[0]-1]), \
                interpolation='nearest', cmap=FContGridColor.colorMaps[FContGridColor.currentColorMap], \
                norm=matplotlib.colors.Normalize(vmin=imageData.image['clipMin'], vmax=imageData.image['clipMax']))
        self.axes.set_xlabel('RA offset [arcsec]')
        self.axes.set_ylabel('DEC offset [arcsec]')

        if imageData.image['enableCLines']:
            data = LCdata
            
            clev = LContGrid[clineVarName].val
            if LContGridColor[clineVarName].isCurrentSingleColor:
                color = LContGridColor[clineVarName].currentColor
                self.cm_lines = self.axes.contour(imageData.image['crd_x'], np.flipud(imageData.image['crd_y']), data, clev, colors=color)
            else:
                cmap= LContGridColor[clineVarName].colorMaps[LContGridColor[clineVarName].currentColorMap]
                self.cm_lines = self.axes.contour(imageData.image['crd_x'], np.flipud(imageData.image['crd_y']), data, clev, cmap=cmap)

        self.cbar = self.fig.colorbar(self.cm_plot)
        self.cbar.set_clim(imageData.image['clipMin'], imageData.image['clipMax'])
        ticklabels = getp(self.cbar.ax, 'ymajorticklabels') 
        setp(ticklabels, fontsize=self.figureConfigParams['colorbar_fontsize'])
        self.cbar.ax.set_ylabel(cbar_label)
  
        self.cbar.outline.set_color(self.figureConfigParams['fgcolor'])
        self.cbar.ax.tick_params(axis='x', colors=self.figureConfigParams['fgcolor']) 
        self.cbar.ax.tick_params(axis='y', colors=self.figureConfigParams['fgcolor']) 
        self.cbar.ax.xaxis.label.set_color(self.figureConfigParams['fgcolor'])
        self.cbar.ax.yaxis.label.set_color(self.figureConfigParams['fgcolor'])

        self.cbar.ax.tick_params(length=3.5, width=1.0, left='on', right='on')

        self.cbar.ax.hold(True)
        self.cbar.ax.set_adjustable('box')
       
        self.draw() 


class mplFigureImage(QtGui.QWidget):
    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)


        self.canvas = MplCanvasImage()
        self.toolbar = MyToolbar(self.canvas, self)
        self.vbl = QtGui.QVBoxLayout()
        self.vbl.addWidget(self.canvas)
        self.vbl.addWidget(self.toolbar)
        self.setLayout(self.vbl)
        


